/**
 * Copyright 2012-2013 StackMob
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#import <Kiwi/Kiwi.h>
#import "StackMob.h"
#import "SMIntegrationTestHelpers.h"
#import "SMCoreDataIntegrationTestHelpers.h"
#import "Person.h"
#import "SMTestProperties.h"

SPEC_BEGIN(LocalReadCacheSpec)


pending_(@"fetch request that errors returns properly", ^{
    
});
pending_(@"changing the default expand depth on queries", ^{
    
});
pending_(@"sending request options that overwrite the cache policy", ^{
    
});
pending_(@"sending request options that overwrite the expand depth", ^{
    
});
pending_(@"completion block when purging is sucessful", ^{
    
});


describe(@"Cache Results option", ^{
    __block SMTestProperties *testProperties;
    beforeEach(^{
        SM_CACHE_ENABLED = YES;
        testProperties = [[SMTestProperties alloc] init];
    });
    afterEach(^{
        [testProperties.cds setFetchPolicy:SMFetchPolicyNetworkOnly];
        NSFetchRequest *request = [[NSFetchRequest alloc] initWithEntityName:@"Todo"];
        NSError *error = nil;
        NSArray *array = [testProperties.moc executeFetchRequestAndWait:request error:&error];
        
        [error shouldBeNil];
        [array enumerateObjectsUsingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
            [testProperties.moc deleteObject:obj];
        }];
        
        if ([testProperties.moc hasChanges]) {
            error = nil;
            [testProperties.moc saveAndWait:&error];
            [error shouldBeNil];
        }
        
        SM_CACHE_ENABLED = NO;
    });
    it(@"default is YES works saves", ^{
        for (int i=0; i < 10; i++) {
            NSManagedObject *todo = [NSEntityDescription insertNewObjectForEntityForName:@"Todo" inManagedObjectContext:testProperties.moc];
            [todo assignObjectId];
        }
        NSError *error = nil;
        BOOL success = [testProperties.moc saveAndWait:&error];
        [[theValue(success) should] beYes];
        [error shouldBeNil];
        
        sleep(SLEEP_TIME);
        
        NSFetchRequest *request = [[NSFetchRequest alloc] initWithEntityName:@"Todo"];
        [testProperties.cds setFetchPolicy:SMFetchPolicyCacheOnly];
        error = nil;
        NSArray *array = [testProperties.moc executeFetchRequestAndWait:request returnManagedObjectIDs:YES error:&error];
        
        [error shouldBeNil];
        [[array should] haveCountOf:10];
        
    });
    it(@"setting cacheResults to NO works saves", ^{
        for (int i=0; i < 10; i++) {
            NSManagedObject *todo = [NSEntityDescription insertNewObjectForEntityForName:@"Todo" inManagedObjectContext:testProperties.moc];
            [todo assignObjectId];
        }
        SMRequestOptions *options = [SMRequestOptions optionsWithCacheResults:NO];
        [[theValue(options.fetchPolicySet) should] beNo];
        NSError *error = nil;
        BOOL success = [testProperties.moc saveAndWait:&error options:options];
        [[theValue(success) should] beYes];
        [error shouldBeNil];
        
        NSFetchRequest *request = [[NSFetchRequest alloc] initWithEntityName:@"Todo"];
        [testProperties.cds setFetchPolicy:SMFetchPolicyCacheOnly];
        error = nil;
        NSArray *array = [testProperties.moc executeFetchRequestAndWait:request returnManagedObjectIDs:YES error:&error];
        
        [error shouldBeNil];
        [[array should] haveCountOf:0];
        
    });
    it(@"default is YES works fetches", ^{
        dispatch_group_t group = dispatch_group_create();
        dispatch_queue_t queue = dispatch_queue_create("queue", NULL);
        
        for (int i=0; i < 10; i++) {
            dispatch_group_enter(group);
            [[testProperties.client dataStore] createObject:[NSDictionary dictionaryWithObjectsAndKeys:@"todo", @"title", nil] inSchema:@"todo" options:[SMRequestOptions options] successCallbackQueue:queue failureCallbackQueue:queue onSuccess:^(NSDictionary *theObject, NSString *schema) {
                dispatch_group_leave(group);
            } onFailure:^(NSError *error, NSDictionary *theObject, NSString *schema) {
                [error shouldBeNil];
                dispatch_group_leave(group);
            }];
        }
        
        dispatch_group_wait(group, DISPATCH_TIME_FOREVER);
        
        sleep(SLEEP_TIME);
        
        NSFetchRequest *request1 = [[NSFetchRequest alloc] initWithEntityName:@"Todo"];
        [testProperties.cds setFetchPolicy:SMFetchPolicyNetworkOnly];
        NSError *error = nil;
        NSArray *array = [testProperties.moc executeFetchRequestAndWait:request1 returnManagedObjectIDs:YES error:&error];
        
        [error shouldBeNil];
        [[array should] haveCountOf:10];
        
        NSFetchRequest *request2 = [[NSFetchRequest alloc] initWithEntityName:@"Todo"];
        [testProperties.cds setFetchPolicy:SMFetchPolicyCacheOnly];
        error = nil;
        NSArray *array2 = [testProperties.moc executeFetchRequestAndWait:request2 returnManagedObjectIDs:YES error:&error];
        
        [error shouldBeNil];
        [[array2 should] haveCountOf:10];
        
    });

    it(@"setting cache results to NO works fetches", ^{
        dispatch_group_t group = dispatch_group_create();
        dispatch_queue_t queue = dispatch_queue_create("queue", NULL);
        
        for (int i=0; i < 10; i++) {
            dispatch_group_enter(group);
            [[testProperties.client dataStore] createObject:[NSDictionary dictionaryWithObjectsAndKeys:@"todo", @"title", nil] inSchema:@"todo" options:[SMRequestOptions options] successCallbackQueue:queue failureCallbackQueue:queue onSuccess:^(NSDictionary *theObject, NSString *schema) {
                dispatch_group_leave(group);
            } onFailure:^(NSError *error, NSDictionary *theObject, NSString *schema) {
                [error shouldBeNil];
                dispatch_group_leave(group);
            }];
        }
        
        dispatch_group_wait(group, DISPATCH_TIME_FOREVER);
        
        sleep(SLEEP_TIME);
        
        NSFetchRequest *request1 = [[NSFetchRequest alloc] initWithEntityName:@"Todo"];
        [testProperties.cds setFetchPolicy:SMFetchPolicyNetworkOnly];
        SMRequestOptions *fetchOptions = [SMRequestOptions optionsWithCacheResults:NO];
        [[theValue(fetchOptions.fetchPolicySet) should] beNo];
        NSError *error = nil;
        NSArray *array = [testProperties.moc executeFetchRequestAndWait:request1 returnManagedObjectIDs:YES options:fetchOptions error:&error];
        
        [error shouldBeNil];
        [[array should] haveCountOf:10];
        
        NSFetchRequest *request2 = [[NSFetchRequest alloc] initWithEntityName:@"Todo"];
        [testProperties.cds setFetchPolicy:SMFetchPolicyCacheOnly];
        error = nil;
        NSArray *array2 = [testProperties.moc executeFetchRequestAndWait:request2 returnManagedObjectIDs:YES error:&error];
        
        [error shouldBeNil];
        [[array2 should] haveCountOf:0];
        
    });
});


describe(@"Per Request Cache Policy", ^{
    __block SMTestProperties *testProperties;
    beforeAll(^{
        SM_CACHE_ENABLED = YES;
        testProperties = [[SMTestProperties alloc] init];
        
        for (int i=0; i < 10; i++) {
            NSManagedObject *todo = [NSEntityDescription insertNewObjectForEntityForName:@"Todo" inManagedObjectContext:testProperties.moc];
            [todo setValue:[todo assignObjectId] forKey:[todo primaryKeyField]];
        }
        NSError *error = nil;
        BOOL success = [testProperties.moc saveAndWait:&error];
        [[theValue(success) should] beYes];
        [error shouldBeNil];
    });
    afterAll(^{
        [testProperties.cds setFetchPolicy:SMFetchPolicyNetworkOnly];
        NSFetchRequest *request = [[NSFetchRequest alloc] initWithEntityName:@"Todo"];
        NSError *error = nil;
        NSArray *array = [testProperties.moc executeFetchRequestAndWait:request error:&error];
        
        [error shouldBeNil];
        [array enumerateObjectsUsingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
            [testProperties.moc deleteObject:obj];
        }];
        
        if ([testProperties.moc hasChanges]) {
            error = nil;
            [testProperties.moc saveAndWait:&error];
            [error shouldBeNil];
        }
        
        SM_CACHE_ENABLED = NO;
    });
    it(@"not setting policy works, sync", ^{
        
        //[[[testProperties.client.session oauthClientWithHTTPS:NO] should] receive:@selector(enqueueBatchOfHTTPRequestOperations:completionBlockQueue:progressBlock:completionBlock:) withCount:0];
        
        [testProperties.cds setFetchPolicy:SMFetchPolicyCacheOnly];
        NSFetchRequest *request = [[NSFetchRequest alloc] initWithEntityName:@"Todo"];
        NSError *error = nil;
        NSArray *array = [testProperties.moc executeFetchRequestAndWait:request returnManagedObjectIDs:YES error:&error];
        
        [error shouldBeNil];
        [[array should] haveCountOf:10];
    });
    it(@"setting request policy works, sync", ^{
        
        [testProperties.cds setFetchPolicy:SMFetchPolicyNetworkOnly];
        
        //[[[testProperties.client.session oauthClientWithHTTPS:NO] should] receive:@selector(enqueueBatchOfHTTPRequestOperations:completionBlockQueue:progressBlock:completionBlock:) withCount:0];
        
        NSFetchRequest *request = [[NSFetchRequest alloc] initWithEntityName:@"Todo"];
        SMRequestOptions *options = [SMRequestOptions optionsWithFetchPolicy:SMFetchPolicyCacheOnly];
        
        [[theValue(options.fetchPolicySet) should] beYes];
        NSError *error = nil;
        NSArray *array = [testProperties.moc executeFetchRequestAndWait:request returnManagedObjectIDs:YES options:options error:&error];
        
        [error shouldBeNil];
        [[array should] haveCountOf:10];
    });
    it(@"not setting policy works, async", ^{
        
        dispatch_group_t group = dispatch_group_create();
        dispatch_queue_t queue = dispatch_queue_create("queue", NULL);
        
        //[[[testProperties.client.session oauthClientWithHTTPS:NO] should] receive:@selector(enqueueBatchOfHTTPRequestOperations:completionBlockQueue:progressBlock:completionBlock:) withCount:0];
        
        [testProperties.cds setFetchPolicy:SMFetchPolicyCacheOnly];
        NSFetchRequest *request = [[NSFetchRequest alloc] initWithEntityName:@"Todo"];
        
        dispatch_group_enter(group);
        [testProperties.moc executeFetchRequest:request returnManagedObjectIDs:YES successCallbackQueue:queue failureCallbackQueue:queue onSuccess:^(NSArray *results) {
            [[results should] haveCountOf:10];
            dispatch_group_leave(group);
        } onFailure:^(NSError *error) {
            [error shouldBeNil];
            dispatch_group_leave(group);
        }];
        
        dispatch_group_wait(group, DISPATCH_TIME_FOREVER);
        
    });
    it(@"setting request policy works, async", ^{
        
        dispatch_group_t group = dispatch_group_create();
        dispatch_queue_t queue = dispatch_queue_create("queue", NULL);
        
        [testProperties.cds setFetchPolicy:SMFetchPolicyNetworkOnly];
        
        //[[[testProperties.client.session oauthClientWithHTTPS:NO] should] receive:@selector(enqueueBatchOfHTTPRequestOperations:completionBlockQueue:progressBlock:completionBlock:) withCount:0];
        
        NSFetchRequest *request = [[NSFetchRequest alloc] initWithEntityName:@"Todo"];
        SMRequestOptions *options = [SMRequestOptions options];
        options.fetchPolicy = SMFetchPolicyCacheOnly;
        
        [[theValue(options.fetchPolicySet) should] beYes];
        
        dispatch_group_enter(group);
        [testProperties.moc executeFetchRequest:request returnManagedObjectIDs:YES successCallbackQueue:queue failureCallbackQueue:queue options:options onSuccess:^(NSArray *results) {
            [[results should] haveCountOf:10];
            dispatch_group_leave(group);
        } onFailure:^(NSError *error) {
            [error shouldBeNil];
            dispatch_group_leave(group);
        }];
        
        dispatch_group_wait(group, DISPATCH_TIME_FOREVER);
    });
});

describe(@"Successful fetching replaces equivalent results of fetching from cache", ^{
    __block SMTestProperties *testProperties = nil;
    __block NSArray *fixturesToLoad;
    __block NSDictionary *fixtures;
    beforeEach(^{
        SM_CACHE_ENABLED = YES;
        //SM_CORE_DATA_DEBUG = YES;
        testProperties = [[SMTestProperties alloc] init];
        fixturesToLoad = [NSArray arrayWithObjects:@"person", nil];
        fixtures = [SMIntegrationTestHelpers loadFixturesNamed:fixturesToLoad];
        
        sleep(SLEEP_TIME);
    });
    afterEach(^{
        [testProperties.cds setFetchPolicy:SMFetchPolicyNetworkOnly];
        [SMIntegrationTestHelpers destroyAllForFixturesNamed:fixturesToLoad];
        SM_CACHE_ENABLED = NO;
    });
    it(@"works", ^{
        // Add another Matt
        Person *anotherMatt = [NSEntityDescription insertNewObjectForEntityForName:@"Person" inManagedObjectContext:testProperties.moc];
        NSString *mattObjectID = [anotherMatt assignObjectId];
        [anotherMatt setValue:mattObjectID forKey:[anotherMatt primaryKeyField]];
        [anotherMatt setValue:@"Matt" forKey:@"first_name"];
        
        // save them to the server
        [SMCoreDataIntegrationTestHelpers executeSynchronousSave:testProperties.moc withBlock:^(NSError *error) {
            [error shouldBeNil];
        }];
        
        
        [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:testProperties.moc withRequest:[SMCoreDataIntegrationTestHelpers makePersonFetchRequest:[NSPredicate predicateWithFormat:@"first_name == 'Matt'"] context:testProperties.moc] andBlock:^(NSArray *results, NSError *error) {
            [[theValue([results count]) should] equal:theValue(2)];
            [error shouldBeNil];
        }];
        
        // The Number of things cached should be 2
        __block NSDictionary *lcMapResults = nil;
        NSURL *cacheMapURL = [SMCoreDataIntegrationTestHelpers SM_getStoreURLForCacheMapTableWithPublicKey:testProperties.client.publicKey];
        lcMapResults = [SMCoreDataIntegrationTestHelpers getContentsOfFileAtPath:[cacheMapURL path]];
        
        [lcMapResults shouldNotBeNil];
        [[theValue([lcMapResults count]) should] equal:theValue(1)];
        [[theValue([[[lcMapResults allValues] lastObject] count]) should] equal:theValue(2)];
        
        // Delete a Matt from the server
        __block BOOL deleteSuccess = NO;
        syncWithSemaphore(^(dispatch_semaphore_t semaphore) {
            [[testProperties.client dataStore] deleteObjectId:mattObjectID inSchema:@"person" onSuccess:^(NSString *theObjectId, NSString *schema) {
                deleteSuccess = YES;
                syncReturn(semaphore);
            } onFailure:^(NSError *error, NSString *objectId, NSString *schema) {
                deleteSuccess = NO;
                syncReturn(semaphore);
            }];
        });
        
        [[theValue(deleteSuccess) should] beYes];
        
        [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:testProperties.moc withRequest:[SMCoreDataIntegrationTestHelpers makePersonFetchRequest:[NSPredicate predicateWithFormat:@"first_name == 'Matt'"] context:testProperties.moc] andBlock:^(NSArray *results, NSError *error) {
            [[theValue([results count]) should] equal:theValue(1)];
            [error shouldBeNil];
        }];
        
        // The Number of things cached should be 1
        lcMapResults = nil;
        cacheMapURL = [SMCoreDataIntegrationTestHelpers SM_getStoreURLForCacheMapTableWithPublicKey:testProperties.client.publicKey];
        lcMapResults = [SMCoreDataIntegrationTestHelpers getContentsOfFileAtPath:[cacheMapURL path]];
        
        [lcMapResults shouldNotBeNil];
        [[theValue([lcMapResults count]) should] equal:theValue(1)];
        [[theValue([[[lcMapResults allValues] lastObject] count]) should] equal:theValue(1)];
        
        
    });
});

describe(@"Fetch with Cache", ^{
    __block SMTestProperties *testProperties = nil;
    __block NSArray *fixturesToLoad;
    __block NSDictionary *fixtures;
    beforeEach(^{
        SM_CACHE_ENABLED = YES;
        //SM_CORE_DATA_DEBUG = YES;
        testProperties = [[SMTestProperties alloc] init];
        fixturesToLoad = [NSArray arrayWithObjects:@"person", nil];
        fixtures = [SMIntegrationTestHelpers loadFixturesNamed:fixturesToLoad];
        
        sleep(SLEEP_TIME);
    });
    afterEach(^{
        [testProperties.cds setFetchPolicy:SMFetchPolicyNetworkOnly];
        [SMIntegrationTestHelpers destroyAllForFixturesNamed:fixturesToLoad];
        SM_CACHE_ENABLED = NO;
    });
    describe(@"Cache else network logic", ^{
        it(@"behaves properly", ^{
            __block NSArray *fetchResults = nil;
            [testProperties.cds setFetchPolicy:SMFetchPolicyTryCacheElseNetwork];
            
            //[[testProperties.cds should] receive:@selector(performQuery:options:successCallbackQueue:failureCallbackQueue:onSuccess:onFailure:) withCount:1];
            
            [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:testProperties.moc withRequest:[SMCoreDataIntegrationTestHelpers makePersonFetchRequest:nil context:testProperties.moc] andBlock:^(NSArray *results, NSError *error) {
                fetchResults = results;
                [error shouldBeNil];
            }];
            
            [[theValue([fetchResults count]) should] equal:theValue(3)];
            
            
            [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:testProperties.moc withRequest:[SMCoreDataIntegrationTestHelpers makePersonFetchRequest:nil context:testProperties.moc] andBlock:^(NSArray *results, NSError *error) {
                fetchResults = results;
                [error shouldBeNil];
            }];
            
            [[theValue([fetchResults count]) should] equal:theValue(3)];
            
            
        });
    });
    describe(@"General Fetch Flow", ^{
        it(@"cache enabled, returned objects are saved into local cache without error", ^{
            __block NSArray *smResults = nil;
            __block NSArray *lcResults = nil;
            __block NSDictionary *lcMapResults = nil;
            
            [[theValue([testProperties.cds fetchPolicy]) should] equal:theValue(SMFetchPolicyNetworkOnly)];
            
            [testProperties.cds setFetchPolicy:SMFetchPolicyTryCacheElseNetwork];
            
            [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:testProperties.moc withRequest:[SMCoreDataIntegrationTestHelpers makePersonFetchRequest:nil context:testProperties.moc] andBlock:^(NSArray *results, NSError *error) {
                smResults = results;
                [error shouldBeNil];
            }];
            
            [testProperties.cds setFetchPolicy:SMFetchPolicyNetworkOnly];
            
            [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:testProperties.moc withRequest:[SMCoreDataIntegrationTestHelpers makePersonFetchRequest:nil context:testProperties.moc] andBlock:^(NSArray *results, NSError *error) {
                smResults = results;
                [error shouldBeNil];
            }];
            
            // Cache should have three entries
            NSURL *cacheMapURL = [SMCoreDataIntegrationTestHelpers SM_getStoreURLForCacheMapTableWithPublicKey:testProperties.client.publicKey];
            lcMapResults = [SMCoreDataIntegrationTestHelpers getContentsOfFileAtPath:[cacheMapURL path]];
            
            [lcMapResults shouldNotBeNil];
            [[theValue([lcMapResults count]) should] equal:theValue(1)];
            [[theValue([[[lcMapResults allValues] lastObject] count]) should] equal:theValue(3)];
            
            // let's try again to see that fetching same objects multiple times is smooth
            [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:testProperties.moc withRequest:[SMCoreDataIntegrationTestHelpers makePersonFetchRequest:nil context:testProperties.moc] andBlock:^(NSArray *results, NSError *error) {
                smResults = results;
            }];
            
            lcMapResults = [SMCoreDataIntegrationTestHelpers getContentsOfFileAtPath:[cacheMapURL path]];
            
            [lcMapResults shouldNotBeNil];
            [[theValue([lcMapResults count]) should] equal:theValue(1)];
            [[theValue([[[lcMapResults allValues] lastObject] count]) should] equal:theValue(3)];
            
            // Let's 100% check
            [testProperties.cds setFetchPolicy:SMFetchPolicyCacheOnly];
            
            [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:testProperties.moc withRequest:[SMCoreDataIntegrationTestHelpers makePersonFetchRequest:nil context:testProperties.moc] andBlock:^(NSArray *results, NSError *error) {
                lcResults = results;
            }];
            
            [lcResults enumerateObjectsUsingBlock:^(id managedObjectID, NSUInteger idx, BOOL *stop) {
                [[theValue([smResults indexOfObject:managedObjectID] != NSNotFound) should] beYes];
            }];
            
            // TODO Add in here to compare fields
            
            
            // Should also give us the same results
            [testProperties.cds setFetchPolicy:SMFetchPolicyTryCacheElseNetwork];
            
            [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:testProperties.moc withRequest:[SMCoreDataIntegrationTestHelpers makePersonFetchRequest:nil context:testProperties.moc] andBlock:^(NSArray *results, NSError *error) {
                lcResults = results;
            }];
            
            [lcResults enumerateObjectsUsingBlock:^(id managedObjectID, NSUInteger idx, BOOL *stop) {
                [[theValue([smResults indexOfObject:managedObjectID] != NSNotFound) should] beYes];
            }];
            
        });
        
    });
    
    describe(@"when in memory differs from lc", ^{
        it(@"handles correctly", ^{
            __block NSString *firstName = nil;
            __block NSString *personId = nil;
            [[theValue([testProperties.cds fetchPolicy]) should] equal:theValue(SMFetchPolicyNetworkOnly)];
            
            // fetch new object, which will fault
            [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:testProperties.moc withRequest:[SMCoreDataIntegrationTestHelpers makePersonFetchRequest:[NSPredicate predicateWithFormat:@"first_name == 'Jon'"] context:testProperties.moc] andBlock:^(NSArray *results, NSError *error) {
                [[theValue([results count]) should] equal:theValue(1)];
                // grab its values into memory
                firstName = [[results objectAtIndex:0] valueForKey:@"first_name"];
                personId = [[results objectAtIndex:0] valueForKey:@"person_id"];
            }];
            
            // update object on the server
            NSDictionary *dict = [NSDictionary dictionaryWithObjectsAndKeys:@"Bob", @"first_name", nil];
            syncWithSemaphore(^(dispatch_semaphore_t semaphore) {
                [[testProperties.client dataStore] updateObjectWithId:personId inSchema:@"person" update:dict onSuccess:^(NSDictionary *theObject, NSString *schema) {
                    syncReturn(semaphore);
                } onFailure:^(NSError *error, NSDictionary *object, NSString *schema) {
                    [error shouldBeNil];
                    syncReturn(semaphore);
                }];
            });
            
            sleep(SLEEP_TIME);
            
            
            __block NSString *smFirstName = nil;
            
            // fetch object from stackmob
            [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:testProperties.moc withRequest:[SMCoreDataIntegrationTestHelpers makePersonFetchRequest:[NSPredicate predicateWithFormat:@"first_name == 'Bob'"] context:testProperties.moc] andBlock:^(NSArray *results, NSError *error) {
                [[theValue([results count]) should] equal:theValue(1)];
                
                // this should be the in memory value
                smFirstName = [results objectAtIndex:0] ? [[results objectAtIndex:0] valueForKey:@"first_name"] : nil;
            }];
            
            // reset memory
            [testProperties.moc reset];
            
            [testProperties.cds setFetchPolicy:SMFetchPolicyCacheOnly];
            
            // fetch from LC
            __block NSString *lcFirstName = nil;
            [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:testProperties.moc withRequest:[SMCoreDataIntegrationTestHelpers makePersonFetchRequest:[NSPredicate predicateWithFormat:@"first_name == 'Bob'"] context:testProperties.moc] andBlock:^(NSArray *results, NSError *error) {
                [[theValue([results count]) should] equal:theValue(1)];
                lcFirstName = [[results objectAtIndex:0] valueForKey:@"first_name"];
            }];
            
            [[smFirstName should] equal:lcFirstName];
            
        });
    });
    
    describe(@"relationships and in memory", ^{
        it(@"should handle correctly for relationships as well", ^{
            __block NSString *firstName = nil;
            __block NSString *personId = nil;
            __block NSManagedObject *jonObject = nil;
            __block NSString *superpowerId = nil;
            
            [[theValue([testProperties.cds fetchPolicy]) should] equal:theValue(SMFetchPolicyNetworkOnly)];
            //SM_CORE_DATA_DEBUG = YES;
            // fetch new object, which will fault
            [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:testProperties.moc withRequest:[SMCoreDataIntegrationTestHelpers makePersonFetchRequest:[NSPredicate predicateWithFormat:@"first_name == 'Jon'"] context:testProperties.moc] andBlock:^(NSArray *results, NSError *error) {
                [[theValue([results count]) should] equal:theValue(1)];
                // grab its values into memory
                jonObject = [results objectAtIndex:0];
                firstName = [[results objectAtIndex:0] valueForKey:@"first_name"];
                personId = [[results objectAtIndex:0] valueForKey:@"person_id"];
            }];
            
            // add some related objects
            NSManagedObject *superpower = [NSEntityDescription insertNewObjectForEntityForName:@"Superpower" inManagedObjectContext:testProperties.moc];
            superpowerId = [superpower assignObjectId];
            [superpower setValue:superpowerId forKey:[superpower primaryKeyField]];
            [superpower setValue:@"superpower" forKey:@"name"];
            
            
            NSManagedObject *interest1 = [NSEntityDescription insertNewObjectForEntityForName:@"Interest" inManagedObjectContext:testProperties.moc];
            [interest1 setValue:[interest1 assignObjectId] forKey:[interest1 primaryKeyField]];
            [interest1 setValue:@"interest1" forKey:@"name"];
            
            NSManagedObject *interest2 = [NSEntityDescription insertNewObjectForEntityForName:@"Interest" inManagedObjectContext:testProperties.moc];
            [interest2 setValue:[interest2 assignObjectId] forKey:[interest2 primaryKeyField]];
            [interest2 setValue:@"interest2" forKey:@"name"];
            
            
            // save them to the server
            [SMCoreDataIntegrationTestHelpers executeSynchronousSave:testProperties.moc withBlock:^(NSError *error) {
                [error shouldBeNil];
            }];
            
            sleep(SLEEP_TIME + SLEEP_TIME);
            
            // relate and save
            [jonObject setValue:superpower forKey:@"superpower"];
            [jonObject setValue:[NSSet setWithObjects:interest1, interest2, nil] forKey:@"interests"];
            
            [SMCoreDataIntegrationTestHelpers executeSynchronousSave:testProperties.moc withBlock:^(NSError *error) {
                [error shouldBeNil];
            }];
            
            sleep(SLEEP_TIME + SLEEP_TIME);
            
            // fetch all that stuff
            [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:testProperties.moc withRequest:[SMCoreDataIntegrationTestHelpers makePersonFetchRequest:[NSPredicate predicateWithFormat:@"first_name == 'Jon'"] context:testProperties.moc] andBlock:^(NSArray *results, NSError *error) {
                [[theValue([results count]) should] equal:theValue(1)];
                // grab its values into memory
                firstName = [[results objectAtIndex:0] valueForKey:@"first_name"];
                personId = [[results objectAtIndex:0] valueForKey:@"person_id"];
            }];
            
            
            // update object on the server
            NSDictionary *dict = [NSDictionary dictionaryWithObjectsAndKeys:@"Bob", @"first_name", nil];
            syncWithSemaphore(^(dispatch_semaphore_t semaphore) {
                [[testProperties.client dataStore] updateObjectWithId:personId inSchema:@"person" update:dict onSuccess:^(NSDictionary *theObject, NSString *schema) {
                    syncReturn(semaphore);
                } onFailure:^(NSError *error, NSDictionary *object, NSString *schema) {
                    [error shouldBeNil];
                    syncReturn(semaphore);
                }];
            });
            
            sleep(SLEEP_TIME + SLEEP_TIME);
            
            
            __block NSString *smFirstName = nil;
            
            // fetch object from stackmob
            __block NSManagedObject *object = nil;
            __block NSManagedObjectContext *mocToCompare = testProperties.moc;
            [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:testProperties.moc withRequest:[SMCoreDataIntegrationTestHelpers makePersonFetchRequest:[NSPredicate predicateWithFormat:@"first_name == 'Bob'"] context:testProperties.moc] andBlock:^(NSArray *results, NSError *error) {
                [[theValue([results count]) should] equal:theValue(1)];
                object = [results objectAtIndex:0];
            }];
        
            // this should be the in memory value
            smFirstName = [object valueForKey:@"first_name"];
            NSManagedObject *superpowerFromPerson = [object valueForKey:@"superpower"];
            NSString *spID = [superpowerFromPerson valueForKey:@"superpower_id"];
            [[spID should] equal:superpowerId];
            NSSet *interestSet = [object valueForKey:@"interests"];
            NSManagedObject *firstInterest = [interestSet anyObject];
            NSManagedObjectContext *interestMOC = [firstInterest managedObjectContext];
            [[interestMOC should] equal:mocToCompare];
            
            // reset memory
            [testProperties.moc reset];
            [[testProperties.moc parentContext] reset];
            
            [testProperties.cds setFetchPolicy:SMFetchPolicyCacheOnly];
            
            // fetch from LC
            __block NSString *lcFirstName = nil;
            [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:testProperties.moc withRequest:[SMCoreDataIntegrationTestHelpers makePersonFetchRequest:[NSPredicate predicateWithFormat:@"first_name == 'Bob'"] context:testProperties.moc] andBlock:^(NSArray *results, NSError *error) {
                [[theValue([results count]) should] equal:theValue(1)];
                lcFirstName = [[results objectAtIndex:0] valueForKey:@"first_name"];
            }];
            
            [[smFirstName should] equal:lcFirstName];
            
            // delete objects
            
            [testProperties.cds setFetchPolicy:SMFetchPolicyNetworkOnly];
            
            [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:testProperties.moc withRequest:[SMCoreDataIntegrationTestHelpers makeSuperpowerFetchRequest:nil context:testProperties.moc] andBlock:^(NSArray *results, NSError *error) {
                [results enumerateObjectsUsingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
                    [testProperties.moc deleteObject:obj];
                }];
            }];
            [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:testProperties.moc withRequest:[SMCoreDataIntegrationTestHelpers makeInterestFetchRequest:nil context:testProperties.moc] andBlock:^(NSArray *results, NSError *error) {
                [results enumerateObjectsUsingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
                    [testProperties.moc deleteObject:obj];
                }];
            }];
            [SMCoreDataIntegrationTestHelpers executeSynchronousSave:testProperties.moc withBlock:^(NSError *error) {
                [error shouldBeNil];
            }];
            //SM_CORE_DATA_DEBUG = NO;
        });
    });
    
    describe(@"same tests pass when moc is reset each time", ^{
        it(@"returned objects are saved into local cache without error", ^{
            __block NSArray *smResults = nil;
            __block NSArray *lcResults = nil;
            [[theValue([testProperties.cds fetchPolicy]) should] equal:theValue(SMFetchPolicyNetworkOnly)];
            [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:testProperties.moc withRequest:[SMCoreDataIntegrationTestHelpers makePersonFetchRequest:nil context:testProperties.moc] andBlock:^(NSArray *results, NSError *error) {
                smResults = results;
            }];
            
            [testProperties.cds setFetchPolicy:SMFetchPolicyCacheOnly];
            [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:testProperties.moc withRequest:[SMCoreDataIntegrationTestHelpers makePersonFetchRequest:nil context:testProperties.moc] andBlock:^(NSArray *results, NSError *error) {
                lcResults = results;
            }];
            [[theValue([smResults count]) should] equal:theValue([lcResults count])];
            
            // make sure the results match
            [smResults enumerateObjectsUsingBlock:^(id smResultObj, NSUInteger smResultIdx, BOOL *stop) {
                for (NSString *key in [[[NSEntityDescription entityForName:@"Person" inManagedObjectContext:testProperties.moc] attributesByName] allKeys]) {
                    NSManagedObject *smObj = [smResultObj valueForKey:key];
                    NSManagedObject *lcObj = [[lcResults objectAtIndex:smResultIdx] valueForKey:key];
                    [[smObj should] equal:lcObj];
                }
                [[[NSEntityDescription entityForName:@"Person" inManagedObjectContext:testProperties.moc] relationshipsByName] enumerateKeysAndObjectsUsingBlock:^(id relationshipName, id relationshipDescription, BOOL *stopWithRelationships) {
                    if ([relationshipDescription isToMany]) {
                        if ([smResultObj valueForKey:relationshipName] == nil) {
                            [[[lcResults objectAtIndex:smResultIdx] valueForKey:relationshipName] shouldBeNil];
                        } else {
                            [[theValue([[smResultObj valueForKey:relationshipName] count]) should] equal:theValue([[[lcResults objectAtIndex:smResultIdx] valueForKey:relationshipName] count])];
                        }
                    } else {
                        if ([smResultObj valueForKey:relationshipName] == nil) {
                            [[[lcResults objectAtIndex:smResultIdx] valueForKey:relationshipName] shouldBeNil];
                        } else {
                            [[[smResultObj valueForKey:relationshipName] should] equal:[[lcResults objectAtIndex:smResultIdx] valueForKey:relationshipName]];
                        }
                    }
                }];
            }];
            
            // let's try again to see that fetching same objects multiple times is smooth
            [testProperties.moc reset];
            
            [testProperties.cds setFetchPolicy:SMFetchPolicyNetworkOnly];
            [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:testProperties.moc withRequest:[SMCoreDataIntegrationTestHelpers makePersonFetchRequest:nil context:testProperties.moc] andBlock:^(NSArray *results, NSError *error) {
                smResults = results;
            }];
            [testProperties.cds setFetchPolicy:SMFetchPolicyCacheOnly];
            [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:testProperties.moc withRequest:[SMCoreDataIntegrationTestHelpers makePersonFetchRequest:nil context:testProperties.moc] andBlock:^(NSArray *results, NSError *error) {
                lcResults = results;
            }];
            [[theValue([smResults count]) should] equal:theValue([lcResults count])];
            
            // make sure the results match
            [smResults enumerateObjectsUsingBlock:^(id smResultObj, NSUInteger smResultIdx, BOOL *stop) {
                for (NSString *key in [[[NSEntityDescription entityForName:@"Person" inManagedObjectContext:testProperties.moc] attributesByName] allKeys]) {
                    NSManagedObject *smObj = [smResultObj valueForKey:key];
                    NSManagedObject *lcObj = [[lcResults objectAtIndex:smResultIdx] valueForKey:key];
                    [[smObj should] equal:lcObj];
                }
                [[[NSEntityDescription entityForName:@"Person" inManagedObjectContext:testProperties.moc] relationshipsByName] enumerateKeysAndObjectsUsingBlock:^(id relationshipName, id relationshipDescription, BOOL *stopWithRelationships) {
                    if ([relationshipDescription isToMany]) {
                        if ([smResultObj valueForKey:relationshipName] == nil) {
                            [[[lcResults objectAtIndex:smResultIdx] valueForKey:relationshipName] shouldBeNil];
                        } else {
                            [[theValue([[smResultObj valueForKey:relationshipName] count]) should] equal:theValue([[[lcResults objectAtIndex:smResultIdx] valueForKey:relationshipName] count])];
                        }
                    } else {
                        if ([smResultObj valueForKey:relationshipName] == nil) {
                            [[[lcResults objectAtIndex:smResultIdx] valueForKey:relationshipName] shouldBeNil];
                        } else {
                            [[[smResultObj valueForKey:relationshipName] should] equal:[[lcResults objectAtIndex:smResultIdx] valueForKey:relationshipName]];
                        }
                    }
                }];
            }];
            
            [testProperties.moc reset];
            
            // update to object will work on correct MOC
            [testProperties.cds setFetchPolicy:SMFetchPolicyNetworkOnly];
            NSPredicate *jonPredicate = [NSPredicate predicateWithFormat:@"first_name == 'Jon'"];
            __block NSArray *jonFetchResults = nil;
            [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:testProperties.moc withRequest:[SMCoreDataIntegrationTestHelpers makePersonFetchRequest:jonPredicate context:testProperties.moc] andBlock:^(NSArray *results, NSError *error) {
                [[theValue([results count]) should] equal:theValue(1)];
                jonFetchResults = results;
            }];
            NSManagedObject *object = [jonFetchResults objectAtIndex:0];
            [[[object valueForKey:@"first_name"] should] equal:@"Jon"];
            
            [object setValue:@"Ty" forKey:@"first_name"];
            [SMCoreDataIntegrationTestHelpers executeSynchronousSave:testProperties.moc withBlock:^(NSError *error) {
                [error shouldBeNil];
            }];
            
            // Should then be able to fetch updated object, go offline and fetch updated again
            [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:testProperties.moc withRequest:[SMCoreDataIntegrationTestHelpers makePersonFetchRequest:[NSPredicate predicateWithFormat:@"first_name == 'Ty'"] context:testProperties.moc] andBlock:^(NSArray *results, NSError *error) {
                [[theValue([results count]) should] equal:theValue(1)];
                NSManagedObject *tyObject = [results objectAtIndex:0];
                [[[tyObject valueForKey:@"first_name"] should] equal:@"Ty"];
            }];
            
            [testProperties.moc reset];
            
            [testProperties.cds setFetchPolicy:SMFetchPolicyCacheOnly];
            
            [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:testProperties.moc withRequest:[SMCoreDataIntegrationTestHelpers makePersonFetchRequest:[NSPredicate predicateWithFormat:@"first_name == 'Ty'"] context:testProperties.moc] andBlock:^(NSArray *results, NSError *error) {
                [[theValue([results count]) should] equal:theValue(1)];
                NSManagedObject *tyObject = [results objectAtIndex:0];
                [[[tyObject valueForKey:@"first_name"] should] equal:@"Ty"];
            }];
        });
    });
    
    describe(@"newValuesForRelationship offline testing, To-One", ^{
        it(@"to-one null relationship returns null", ^{
            __block NSManagedObject *jonObject = nil;
            
            [testProperties.cds setFetchPolicy:SMFetchPolicyNetworkOnly];
            
            [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:testProperties.moc withRequest:[SMCoreDataIntegrationTestHelpers makePersonFetchRequest:[NSPredicate predicateWithFormat:@"first_name == 'Jon'"] context:testProperties.moc] andBlock:^(NSArray *results, NSError *error) {
                [[theValue([results count]) should] equal:theValue(1)];
                jonObject = [results objectAtIndex:0];
                NSManagedObject *nullSuperpower = [jonObject valueForKey:@"superpower"];
                [nullSuperpower shouldBeNil];
            }];
        });
        
        it(@"to-one relationship fault fill without internet when related object has NOT been previously fetched remains a fault", ^{
            __block NSManagedObject *jonObject = nil;
            __block NSString *superpowerId = nil;
            // go online
            [testProperties.cds setFetchPolicy:SMFetchPolicyNetworkOnly];
            
            // fetch new object, which will fault
            [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:testProperties.moc withRequest:[SMCoreDataIntegrationTestHelpers makePersonFetchRequest:[NSPredicate predicateWithFormat:@"first_name == 'Jon'"] context:testProperties.moc] andBlock:^(NSArray *results, NSError *error) {
                [[theValue([results count]) should] equal:theValue(1)];
                jonObject = [results objectAtIndex:0];
                NSManagedObject *nullSuperpower = [jonObject valueForKey:@"superpower"];
                [nullSuperpower shouldBeNil];
            }];

            // add some related objects
            NSManagedObject *superpower = [NSEntityDescription insertNewObjectForEntityForName:@"Superpower" inManagedObjectContext:testProperties.moc];
            superpowerId = [superpower assignObjectId];
            [superpower setValue:superpowerId forKey:[superpower primaryKeyField]];
            [superpower setValue:@"superpower" forKey:@"name"];
            
            // save them to the server
            [SMCoreDataIntegrationTestHelpers executeSynchronousSave:testProperties.moc withBlock:^(NSError *error) {
                [error shouldBeNil];
            }];
            
            sleep(SLEEP_TIME);
            
            // relate and save
            [jonObject setValue:superpower forKey:@"superpower"];
            
            [SMCoreDataIntegrationTestHelpers executeSynchronousSave:testProperties.moc withBlock:^(NSError *error) {
                [error shouldBeNil];
            }];
            
            sleep(SLEEP_TIME);
            
            [testProperties.moc reset];
            [testProperties.moc.parentContext reset];
            
            [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:testProperties.moc withRequest:[SMCoreDataIntegrationTestHelpers makePersonFetchRequest:[NSPredicate predicateWithFormat:@"first_name == 'Jon'"] context:testProperties.moc] andBlock:^(NSArray *results, NSError *error) {
                [[theValue([results count]) should] equal:theValue(1)];
                jonObject = [results objectAtIndex:0];
            }];
            
            [testProperties.cds setFetchPolicy:SMFetchPolicyCacheOnly];
            
            NSString *name = [jonObject valueForKey:@"first_name"];
            [[name should] equal:@"Jon"];
            NSError *anError = nil;
            NSManagedObject *jonSuperpower = [jonObject valueForRelationshipKey:@"superpower" error:&anError];
            
            NSString *jonSuperpowerName = [jonSuperpower valueForKey:@"name"];
            [[jonSuperpowerName should] equal:@"superpower"];
            
            // delete objects
            [testProperties.cds setFetchPolicy:SMFetchPolicyNetworkOnly];
            [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:testProperties.moc withRequest:[SMCoreDataIntegrationTestHelpers makeSuperpowerFetchRequest:nil context:testProperties.moc] andBlock:^(NSArray *results, NSError *error) {
                [results enumerateObjectsUsingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
                    [testProperties.moc deleteObject:obj];
                }];
            }];
            [SMCoreDataIntegrationTestHelpers executeSynchronousSave:testProperties.moc withBlock:^(NSError *error) {
                [error shouldBeNil];
            }];
            
            sleep(SLEEP_TIME);
        });
        
        it(@"to-one relationship fault fill without internet when related object has been previously fetched returns properly", ^{
            __block NSManagedObject *jonObject = nil;
            __block NSString *superpowerId = nil;
            [testProperties.cds setFetchPolicy:SMFetchPolicyNetworkOnly];
            
            // fetch new object, which will fault
            [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:testProperties.moc withRequest:[SMCoreDataIntegrationTestHelpers makePersonFetchRequest:[NSPredicate predicateWithFormat:@"first_name == 'Jon'"] context:testProperties.moc] andBlock:^(NSArray *results, NSError *error) {
                [[theValue([results count]) should] equal:theValue(1)];
                jonObject = [results objectAtIndex:0];
                NSManagedObject *nullSuperpower = [jonObject valueForKey:@"superpower"];
                [nullSuperpower shouldBeNil];
            }];
            
            // add some related objects
            NSManagedObject *superpower = [NSEntityDescription insertNewObjectForEntityForName:@"Superpower" inManagedObjectContext:testProperties.moc];
            superpowerId = [superpower assignObjectId];
            [superpower setValue:superpowerId forKey:[superpower primaryKeyField]];
            [superpower setValue:@"superpower" forKey:@"name"];
            
            // save them to the server
            [SMCoreDataIntegrationTestHelpers executeSynchronousSave:testProperties.moc withBlock:^(NSError *error) {
                [error shouldBeNil];
            }];
            
            sleep(SLEEP_TIME);
            
            // relate and save
            [jonObject setValue:superpower forKey:@"superpower"];
            
            [SMCoreDataIntegrationTestHelpers executeSynchronousSave:testProperties.moc withBlock:^(NSError *error) {
                [error shouldBeNil];
            }];
            
            sleep(SLEEP_TIME);
            
            [testProperties.moc reset];
            
            [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:testProperties.moc withRequest:[SMCoreDataIntegrationTestHelpers makePersonFetchRequest:[NSPredicate predicateWithFormat:@"first_name == 'Jon'"] context:testProperties.moc] andBlock:^(NSArray *results, NSError *error) {
                [[theValue([results count]) should] equal:theValue(1)];
                jonObject = [results objectAtIndex:0];
            }];
            
            [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:testProperties.moc withRequest:[SMCoreDataIntegrationTestHelpers makeSuperpowerFetchRequest:[NSPredicate predicateWithFormat:@"name == 'superpower'"] context:testProperties.moc] andBlock:^(NSArray *results, NSError *error) {
                [[theValue([results count]) should] equal:theValue(1)];
            }];
            
            [testProperties.cds setFetchPolicy:SMFetchPolicyCacheOnly];
            
            NSManagedObject *jonSuperpower = nil;
            @try {
                jonSuperpower = [jonObject valueForKey:@"superpower"];
            }
            @catch (NSException *exception) {
                [exception shouldBeNil];
            }
            
            [jonSuperpower shouldNotBeNil];
            [[[jonSuperpower valueForKey:@"name"] should] equal:@"superpower"];
            
            [testProperties.cds setFetchPolicy:SMFetchPolicyNetworkOnly];
            [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:testProperties.moc withRequest:[SMCoreDataIntegrationTestHelpers makeSuperpowerFetchRequest:nil context:testProperties.moc] andBlock:^(NSArray *results, NSError *error) {
                [results enumerateObjectsUsingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
                    [testProperties.moc deleteObject:obj];
                }];
            }];
            [SMCoreDataIntegrationTestHelpers executeSynchronousSave:testProperties.moc withBlock:^(NSError *error) {
                [error shouldBeNil];
            }];
            
            sleep(SLEEP_TIME);
        });
        
        it(@"to-one relationship fault fill with internet returns related object and caches correctly", ^{
            __block NSManagedObject *jonObject = nil;
            __block NSString *superpowerId = nil;
           [testProperties.cds setFetchPolicy:SMFetchPolicyNetworkOnly];
            
            // fetch new object, which will fault
            [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:testProperties.moc withRequest:[SMCoreDataIntegrationTestHelpers makePersonFetchRequest:[NSPredicate predicateWithFormat:@"first_name == 'Jon'"] context:testProperties.moc] andBlock:^(NSArray *results, NSError *error) {
                [[theValue([results count]) should] equal:theValue(1)];
                jonObject = [results objectAtIndex:0];
                NSManagedObject *nullSuperpower = [jonObject valueForKey:@"superpower"];
                [nullSuperpower shouldBeNil];
            }];
            
            // add some related objects
            NSManagedObject *superpower = [NSEntityDescription insertNewObjectForEntityForName:@"Superpower" inManagedObjectContext:testProperties.moc];
            superpowerId = [superpower assignObjectId];
            [superpower setValue:superpowerId forKey:[superpower primaryKeyField]];
            [superpower setValue:@"superpower" forKey:@"name"];
            
            // save them to the server
            [SMCoreDataIntegrationTestHelpers executeSynchronousSave:testProperties.moc withBlock:^(NSError *error) {
                [error shouldBeNil];
            }];
            
            sleep(SLEEP_TIME);
            
            // relate and save
            [jonObject setValue:superpower forKey:@"superpower"];
            
            [SMCoreDataIntegrationTestHelpers executeSynchronousSave:testProperties.moc withBlock:^(NSError *error) {
                [error shouldBeNil];
            }];
            
            sleep(SLEEP_TIME);
            
            [testProperties.moc reset];
            
            [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:testProperties.moc withRequest:[SMCoreDataIntegrationTestHelpers makePersonFetchRequest:[NSPredicate predicateWithFormat:@"first_name == 'Jon'"] context:testProperties.moc] andBlock:^(NSArray *results, NSError *error) {
                [[theValue([results count]) should] equal:theValue(1)];
                jonObject = [results objectAtIndex:0];
            }];
            
            NSManagedObject *jonSuperpower = [jonObject valueForKey:@"superpower"];

            [jonSuperpower shouldNotBeNil];
            
            // should be able to fetch jonSuperpower after moc was reset
            NSString *superpowerName = [[jonObject valueForKey:@"superpower"] valueForKey:@"name"];
            [[superpowerName should] equal:@"superpower"];
            
            // We can then clear the moc, go offline and fetch both items
            [testProperties.cds setFetchPolicy:SMFetchPolicyCacheOnly];
            [testProperties.moc reset];
            
            [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:testProperties.moc withRequest:[SMCoreDataIntegrationTestHelpers makePersonFetchRequest:[NSPredicate predicateWithFormat:@"first_name == 'Jon'"] context:testProperties.moc] andBlock:^(NSArray *results, NSError *error) {
                [[theValue([results count]) should] equal:theValue(1)];
                jonObject = [results objectAtIndex:0];
            }];
            
            jonSuperpower = [jonObject valueForKey:@"superpower"];
            
            [jonSuperpower shouldNotBeNil];
            
            // should be able to fetch jonSuperpower after moc is reset
            superpowerName = [[jonObject valueForKey:@"superpower"] valueForKey:@"name"];
            [[superpowerName should] equal:@"superpower"];
            
            
            // delete objects
            [testProperties.cds setFetchPolicy:SMFetchPolicyNetworkOnly];
            [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:testProperties.moc withRequest:[SMCoreDataIntegrationTestHelpers makeSuperpowerFetchRequest:nil context:testProperties.moc] andBlock:^(NSArray *results, NSError *error) {
                [results enumerateObjectsUsingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
                    [testProperties.moc deleteObject:obj];
                }];
            }];
            [SMCoreDataIntegrationTestHelpers executeSynchronousSave:testProperties.moc withBlock:^(NSError *error) {
                [error shouldBeNil];
            }];
            
            sleep(SLEEP_TIME);
        });
         
    });
    
    describe(@"newValuesForRelationship offline testing, To-Many", ^{
        
        it(@"to-many null relationship returns empty set", ^{
            __block NSManagedObject *jonObject = nil;
            
            [testProperties.cds setFetchPolicy:SMFetchPolicyNetworkOnly];
            
            [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:testProperties.moc withRequest:[SMCoreDataIntegrationTestHelpers makePersonFetchRequest:[NSPredicate predicateWithFormat:@"first_name == 'Jon'"] context:testProperties.moc] andBlock:^(NSArray *results, NSError *error) {
                [[theValue([results count]) should] equal:theValue(1)];
                jonObject = [results objectAtIndex:0];
                NSSet *interestsSet = [jonObject valueForKey:@"interests"];
                [interestsSet shouldNotBeNil];
                [[theValue([interestsSet count]) should] equal:theValue(0)];
            }];
        });
        
        it(@"To-Many relationship fault fill without internet when related object has NOT been previously fetched remains a fault", ^{
            __block NSManagedObjectContext *testContext = testProperties.moc;
            __block Person *jonObject = nil;
            //SM_CORE_DATA_DEBUG = YES;
            [testProperties.cds setFetchPolicy:SMFetchPolicyNetworkOnly];
            
            // fetch new object, which will fault
            [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:testContext withRequest:[SMCoreDataIntegrationTestHelpers makePersonFetchRequest:[NSPredicate predicateWithFormat:@"first_name == 'Jon'"] context:testContext] andBlock:^(NSArray *results, NSError *error) {
                [[theValue([results count]) should] equal:theValue(1)];
                jonObject = [results objectAtIndex:0];
                NSManagedObject *nullSuperpower = [jonObject valueForKey:@"superpower"];
                [nullSuperpower shouldBeNil];
            }];
            
            // add some related objects
            NSManagedObject *interest1 = [NSEntityDescription insertNewObjectForEntityForName:@"Interest" inManagedObjectContext:testContext];
            [interest1 setValue:[interest1 assignObjectId] forKey:[interest1 primaryKeyField]];
            [interest1 setValue:@"interest1" forKey:@"name"];
            
            NSManagedObject *interest2 = [NSEntityDescription insertNewObjectForEntityForName:@"Interest" inManagedObjectContext:testContext];
            [interest2 setValue:[interest2 assignObjectId] forKey:[interest2 primaryKeyField]];
            [interest2 setValue:@"interest2" forKey:@"name"];
            
            // save them to the server
            [SMCoreDataIntegrationTestHelpers executeSynchronousSave:testContext withBlock:^(NSError *error) {
                [error shouldBeNil];
            }];
            
            sleep(SLEEP_TIME);
            
            // relate and save
            [jonObject addInterests:[NSSet setWithObjects:interest1, interest2, nil]];
            
            [SMCoreDataIntegrationTestHelpers executeSynchronousSave:testContext withBlock:^(NSError *error) {
                [error shouldBeNil];
            }];
            
            sleep(SLEEP_TIME);
            
            [testContext reset];
            [[testContext parentContext] reset];
            
            [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:testContext withRequest:[SMCoreDataIntegrationTestHelpers makePersonFetchRequest:[NSPredicate predicateWithFormat:@"first_name == 'Jon'"] context:testContext] andBlock:^(NSArray *results, NSError *error) {
                [[theValue([results count]) should] equal:theValue(1)];
                jonObject = [results objectAtIndex:0];
            }];
            
            [testProperties.cds setFetchPolicy:SMFetchPolicyCacheOnly];
            
            NSString *jonName = [jonObject valueForKey:@"first_name"];
            [[jonName should] equal:@"Jon"];
            NSSet *jonInterests = [jonObject valueForKey:@"interests"];
            [jonInterests enumerateObjectsUsingBlock:^(id obj, BOOL *stop) {
                NSString *interestName = [obj valueForKey:@"name"];
                if ([interestName isEqualToString:@"interest1"]) {
                    [[interestName should] equal:@"interest1"];
                } else {
                    [[interestName should] equal:@"interest2"];
                }
            }];
            
            // delete objects
            
            [testProperties.cds setFetchPolicy:SMFetchPolicyNetworkOnly];
            
            [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:testContext withRequest:[SMCoreDataIntegrationTestHelpers makeInterestFetchRequest:nil context:testContext] andBlock:^(NSArray *results, NSError *error) {
                [results enumerateObjectsUsingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
                    [testContext deleteObject:obj];
                }];
            }];
            [SMCoreDataIntegrationTestHelpers executeSynchronousSave:testContext withBlock:^(NSError *error) {
                [error shouldBeNil];
            }];
            
            sleep(SLEEP_TIME);
        });
        
        it(@"To-Many relationship fault fill with internet returns related object and caches correctly", ^{
            __block Person *jonObject = nil;
            
            [testProperties.cds setFetchPolicy:SMFetchPolicyNetworkOnly];
            
            // fetch new object, which will fault
            [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:testProperties.moc withRequest:[SMCoreDataIntegrationTestHelpers makePersonFetchRequest:[NSPredicate predicateWithFormat:@"first_name == 'Jon'"] context:testProperties.moc] andBlock:^(NSArray *results, NSError *error) {
                [[theValue([results count]) should] equal:theValue(1)];
                jonObject = [results objectAtIndex:0];
                NSManagedObject *nullSuperpower = [jonObject valueForKey:@"superpower"];
                [nullSuperpower shouldBeNil];
            }];
            
            // add some related objects
            NSManagedObject *interest1 = [NSEntityDescription insertNewObjectForEntityForName:@"Interest" inManagedObjectContext:testProperties.moc];
            [interest1 setValue:[interest1 assignObjectId] forKey:[interest1 primaryKeyField]];
            [interest1 setValue:@"interest1" forKey:@"name"];
            
            NSManagedObject *interest2 = [NSEntityDescription insertNewObjectForEntityForName:@"Interest" inManagedObjectContext:testProperties.moc];
            [interest2 setValue:[interest2 assignObjectId] forKey:[interest2 primaryKeyField]];
            [interest2 setValue:@"interest2" forKey:@"name"];
            
            // save them to the server
            [SMCoreDataIntegrationTestHelpers executeSynchronousSave:testProperties.moc withBlock:^(NSError *error) {
                [error shouldBeNil];
            }];
            
            sleep(SLEEP_TIME);
            
            // relate and save
            [jonObject addInterests:[NSSet setWithObjects:interest1, interest2, nil]];
            
            [SMCoreDataIntegrationTestHelpers executeSynchronousSave:testProperties.moc withBlock:^(NSError *error) {
                [error shouldBeNil];
            }];
            
            sleep(SLEEP_TIME);
            
            [testProperties.moc reset];
            
            [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:testProperties.moc withRequest:[SMCoreDataIntegrationTestHelpers makePersonFetchRequest:[NSPredicate predicateWithFormat:@"first_name == 'Jon'"] context:testProperties.moc] andBlock:^(NSArray *results, NSError *error) {
                [[theValue([results count]) should] equal:theValue(1)];
                jonObject = [results objectAtIndex:0];
            }];
            
            NSArray *jonInterests = [[jonObject valueForKey:@"interests"] allObjects];
            
            // should be able to fetch jonSuperpower after moc was reset
            NSString *interestName = [[jonInterests objectAtIndex:0] valueForKey:@"name"];
            NSArray *interestsArray = [NSArray arrayWithObjects:@"interest1", @"interest2", nil];
            [[interestsArray should] contain:interestName];
            
            // We can then clear the moc, go offline and fetch both items
            [testProperties.cds setFetchPolicy:SMFetchPolicyCacheOnly];
            [testProperties.moc reset];
            
            [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:testProperties.moc withRequest:[SMCoreDataIntegrationTestHelpers makePersonFetchRequest:[NSPredicate predicateWithFormat:@"first_name == 'Jon'"] context:testProperties.moc] andBlock:^(NSArray *results, NSError *error) {
                [[theValue([results count]) should] equal:theValue(1)];
                jonObject = [results objectAtIndex:0];
            }];
            
            jonInterests = [[jonObject valueForKey:@"interests"] allObjects];
            
            // should be able to fetch jonSuperpower after moc was reset
            interestName = [[jonInterests objectAtIndex:0] valueForKey:@"name"];
            interestsArray = [NSArray arrayWithObjects:@"interest1", @"interest2", nil];
            [[interestsArray should] contain:interestName];
            
            
            // delete objects
            [testProperties.cds setFetchPolicy:SMFetchPolicyNetworkOnly];
            [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:testProperties.moc withRequest:[SMCoreDataIntegrationTestHelpers makeInterestFetchRequest:nil context:testProperties.moc] andBlock:^(NSArray *results, NSError *error) {
                [results enumerateObjectsUsingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
                    [testProperties.moc deleteObject:obj];
                }];
            }];
            [SMCoreDataIntegrationTestHelpers executeSynchronousSave:testProperties.moc withBlock:^(NSError *error) {
                [error shouldBeNil];
            }];
            
            sleep(SLEEP_TIME);
        });
     
    });

});


describe(@"Purging the Cache", ^{
    __block SMClient *client = nil;
    __block SMCoreDataStore *cds = nil;
    __block NSManagedObjectContext *moc = nil;
    __block NSArray *fixturesToLoad;
    __block NSDictionary *fixtures;
    beforeEach(^{
        SM_CACHE_ENABLED = YES;
        //SM_CORE_DATA_DEBUG = YES;
        client = [SMIntegrationTestHelpers defaultClient];
        [SMClient setDefaultClient:client];
        [SMCoreDataIntegrationTestHelpers removeSQLiteDatabaseAndMapsWithPublicKey:client.publicKey];
        NSBundle *classBundle = [NSBundle bundleForClass:[self class]];
        NSURL *modelURL = [classBundle URLForResource:@"SMCoreDataIntegrationTest" withExtension:@"momd"];
        NSManagedObjectModel *aModel = [[NSManagedObjectModel alloc] initWithContentsOfURL:modelURL];
        cds = [client coreDataStoreWithManagedObjectModel:aModel];
        moc = [cds contextForCurrentThread];
        fixturesToLoad = [NSArray arrayWithObjects:@"person", nil];
        fixtures = [SMIntegrationTestHelpers loadFixturesNamed:fixturesToLoad];
        
        sleep(SLEEP_TIME);
    });
    afterEach(^{
        [cds setFetchPolicy:SMFetchPolicyNetworkOnly];
        [SMIntegrationTestHelpers destroyAllForFixturesNamed:fixturesToLoad];
        SM_CACHE_ENABLED = NO;
    });
    it(@"Should clear the cache of objects that are deleted", ^{
        __block NSArray *resultfOfFetch = nil;
        [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:moc withRequest:[SMCoreDataIntegrationTestHelpers makePersonFetchRequest:nil context:moc] andBlock:^(NSArray *results, NSError *error) {
            resultfOfFetch = results;
        }];
        
        __block NSDictionary *lcMapResults = nil;
        NSURL *cacheMapURL = [SMCoreDataIntegrationTestHelpers SM_getStoreURLForCacheMapTableWithPublicKey:client.publicKey];
        lcMapResults = [SMCoreDataIntegrationTestHelpers getContentsOfFileAtPath:[cacheMapURL path]];
        
        [lcMapResults shouldNotBeNil];
        [[theValue([lcMapResults count]) should] equal:theValue(1)];
        [[theValue([[[lcMapResults allValues] objectAtIndex:0] count]) should] equal:theValue(3)];
        
        [resultfOfFetch enumerateObjectsUsingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
            [moc deleteObject:obj];
        }];
        
        [SMCoreDataIntegrationTestHelpers executeSynchronousSave:moc withBlock:^(NSError *error) {
            [error shouldBeNil];
        }];
        
        sleep(SLEEP_TIME);
        
        lcMapResults = nil;
        cacheMapURL = [SMCoreDataIntegrationTestHelpers SM_getStoreURLForCacheMapTableWithPublicKey:client.publicKey];
        lcMapResults = [SMCoreDataIntegrationTestHelpers getContentsOfFileAtPath:[cacheMapURL path]];
        
        [lcMapResults shouldNotBeNil];
        [[theValue([lcMapResults count]) should] equal:theValue(0)];
        
        [cds setFetchPolicy:SMFetchPolicyCacheOnly];
        
        [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:moc withRequest:[SMCoreDataIntegrationTestHelpers makePersonFetchRequest:nil context:moc] andBlock:^(NSArray *results, NSError *error) {
            [error shouldBeNil];
            [[theValue([results count]) should] equal:theValue(0)];
        }];
        
    });
    
    it(@"interface for purging the cache of an object", ^{
        __block NSArray *resultfOfFetch = nil;
        [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:moc withRequest:[SMCoreDataIntegrationTestHelpers makePersonFetchRequest:nil context:moc] andBlock:^(NSArray *results, NSError *error) {
            resultfOfFetch = results;
        }];
        
        NSManagedObject *anItem = [resultfOfFetch objectAtIndex:0];
        
        [cds purgeCacheOfManagedObjectID:[anItem objectID]];
        
        sleep(5);
        
        __block NSDictionary *lcMapResults = nil;
        NSURL *cacheMapURL = [SMCoreDataIntegrationTestHelpers SM_getStoreURLForCacheMapTableWithPublicKey:client.publicKey];
        lcMapResults = [SMCoreDataIntegrationTestHelpers getContentsOfFileAtPath:[cacheMapURL path]];
        
        [lcMapResults shouldNotBeNil];
        [[theValue([lcMapResults count]) should] equal:theValue(1)];
        [[theValue([[[lcMapResults allValues] objectAtIndex:0] count]) should] equal:theValue(2)];
        
        // TODO grab actual objects and test with cache objects
        
    });
    it(@"resetting the cache", ^{
        __block NSArray *resultfOfFetch = nil;
        [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:moc withRequest:[SMCoreDataIntegrationTestHelpers makePersonFetchRequest:nil context:moc] andBlock:^(NSArray *results, NSError *error) {
            resultfOfFetch = results;
        }];
        
        [[theValue([resultfOfFetch count]) should] equal:theValue(3)];
        
        __block NSDictionary *lcMapResults = nil;
        NSURL *cacheMapURL = [SMCoreDataIntegrationTestHelpers SM_getStoreURLForCacheMapTableWithPublicKey:client.publicKey];
        lcMapResults = [SMCoreDataIntegrationTestHelpers getContentsOfFileAtPath:[cacheMapURL path]];
        
        [lcMapResults shouldNotBeNil];
        [[theValue([lcMapResults count]) should] equal:theValue(1)];
        [[theValue([[[lcMapResults allValues] objectAtIndex:0] count]) should] equal:theValue(3)];
        
        [cds resetCache];
        
        sleep(5);
        
        [cds setFetchPolicy:SMFetchPolicyCacheOnly];
        
        [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:moc withRequest:[SMCoreDataIntegrationTestHelpers makePersonFetchRequest:nil context:moc] andBlock:^(NSArray *results, NSError *error) {
            resultfOfFetch = results;
        }];
        
        [[theValue([resultfOfFetch count]) should] equal:theValue(0)];
        
        lcMapResults = nil;
        cacheMapURL = [SMCoreDataIntegrationTestHelpers SM_getStoreURLForCacheMapTableWithPublicKey:client.publicKey];
        lcMapResults = [SMCoreDataIntegrationTestHelpers getContentsOfFileAtPath:[cacheMapURL path]];
        
        [lcMapResults shouldNotBeNil];
        [[theValue([lcMapResults count]) should] equal:theValue(0)];
    });
});

describe(@"purging cache of multiple objects at a time", ^{
    __block SMClient *client = nil;
    __block SMCoreDataStore *cds = nil;
    __block NSManagedObjectContext *moc = nil;
    __block NSArray *fixturesToLoad;
    __block NSDictionary *fixtures;
    beforeEach(^{
        SM_CACHE_ENABLED = YES;
        //SM_CORE_DATA_DEBUG = YES;
        client = [SMIntegrationTestHelpers defaultClient];
        [SMClient setDefaultClient:client];
        [SMCoreDataIntegrationTestHelpers removeSQLiteDatabaseAndMapsWithPublicKey:client.publicKey];
        NSBundle *classBundle = [NSBundle bundleForClass:[self class]];
        NSURL *modelURL = [classBundle URLForResource:@"SMCoreDataIntegrationTest" withExtension:@"momd"];
        NSManagedObjectModel *aModel = [[NSManagedObjectModel alloc] initWithContentsOfURL:modelURL];
        cds = [client coreDataStoreWithManagedObjectModel:aModel];
        moc = [cds contextForCurrentThread];
        fixturesToLoad = [NSArray arrayWithObjects:@"person", nil];
        fixtures = [SMIntegrationTestHelpers loadFixturesNamed:fixturesToLoad];
        
        sleep(SLEEP_TIME);
    });
    afterEach(^{
        [cds setFetchPolicy:SMFetchPolicyNetworkOnly];
        SM_CACHE_ENABLED = NO;
    });
    it(@"interface for purging the cache of objects", ^{
        __block NSArray *resultfOfFetch = nil;
        [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:moc withRequest:[SMCoreDataIntegrationTestHelpers makePersonFetchRequest:nil context:moc] andBlock:^(NSArray *results, NSError *error) {
            resultfOfFetch = results;
        }];
        
        [cds purgeCacheOfManagedObjects:resultfOfFetch];
        
        sleep(5);
        
        __block NSDictionary *lcMapResults = nil;
        NSURL *cacheMapURL = [SMCoreDataIntegrationTestHelpers SM_getStoreURLForCacheMapTableWithPublicKey:client.publicKey];
        lcMapResults = [SMCoreDataIntegrationTestHelpers getContentsOfFileAtPath:[cacheMapURL path]];
        
        [lcMapResults shouldNotBeNil];
        [[theValue([lcMapResults count]) should] equal:theValue(0)];
        
        // deleting the objects through CD shouldn't break
        [resultfOfFetch enumerateObjectsUsingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
            [moc deleteObject:obj];
        }];
        
        [SMCoreDataIntegrationTestHelpers executeSynchronousSave:moc withBlock:^(NSError *error) {
            [error shouldBeNil];
        }];
        
        sleep(SLEEP_TIME);
        
    });
    it(@"interface for purging the cache of objects by entity name", ^{
        __block NSArray *resultfOfFetch = nil;
        [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:moc withRequest:[SMCoreDataIntegrationTestHelpers makePersonFetchRequest:nil context:moc] andBlock:^(NSArray *results, NSError *error) {
            resultfOfFetch = results;
        }];
        
        [cds purgeCacheOfObjectsWithEntityName:@"Todo"];
        
        sleep(5);
        
        __block NSDictionary *lcMapResults = nil;
        NSURL *cacheMapURL = [SMCoreDataIntegrationTestHelpers SM_getStoreURLForCacheMapTableWithPublicKey:client.publicKey];
        lcMapResults = [SMCoreDataIntegrationTestHelpers getContentsOfFileAtPath:[cacheMapURL path]];
        
        [lcMapResults shouldNotBeNil];
        [[theValue([lcMapResults count]) should] equal:theValue(1)];
        [[theValue([[[lcMapResults allValues] objectAtIndex:0] count]) should] equal:theValue(3)];
        
        [cds purgeCacheOfObjectsWithEntityName:@"Person"];
        
        sleep(5);
        
        lcMapResults = nil;
        cacheMapURL = [SMCoreDataIntegrationTestHelpers SM_getStoreURLForCacheMapTableWithPublicKey:client.publicKey];
        lcMapResults = [SMCoreDataIntegrationTestHelpers getContentsOfFileAtPath:[cacheMapURL path]];
        
        [lcMapResults shouldNotBeNil];
        [[theValue([lcMapResults count]) should] equal:theValue(0)];
        
        // deleting the objects through CD shouldn't break
        [resultfOfFetch enumerateObjectsUsingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
            [moc deleteObject:obj];
        }];
        
        [SMCoreDataIntegrationTestHelpers executeSynchronousSave:moc withBlock:^(NSError *error) {
            [error shouldBeNil];
        }];
        
        sleep(SLEEP_TIME);
        
    });
});

describe(@"cache references should not be returned during fetches", ^{
    __block SMClient *client = nil;
    __block SMCoreDataStore *cds = nil;
    __block NSManagedObjectContext *moc = nil;
    __block NSArray *fixturesToLoad;
    __block NSDictionary *fixtures;
    beforeEach(^{
        SM_CACHE_ENABLED = YES;
        //SM_CORE_DATA_DEBUG = YES;
        client = [SMIntegrationTestHelpers defaultClient];
        [SMClient setDefaultClient:client];
        [SMCoreDataIntegrationTestHelpers removeSQLiteDatabaseAndMapsWithPublicKey:client.publicKey];
        NSBundle *classBundle = [NSBundle bundleForClass:[self class]];
        NSURL *modelURL = [classBundle URLForResource:@"SMCoreDataIntegrationTest" withExtension:@"momd"];
        NSManagedObjectModel *aModel = [[NSManagedObjectModel alloc] initWithContentsOfURL:modelURL];
        cds = [client coreDataStoreWithManagedObjectModel:aModel];
        moc = [cds contextForCurrentThread];
        fixturesToLoad = [NSArray arrayWithObjects:@"person", nil];
        fixtures = [SMIntegrationTestHelpers loadFixturesNamed:fixturesToLoad];
        
        sleep(SLEEP_TIME);
        
        // Fetch all persons
        __block NSArray *resultsOfFetch = nil;
        [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:moc withRequest:[SMCoreDataIntegrationTestHelpers makePersonFetchRequest:nil context:moc] andBlock:^(NSArray *results, NSError *error) {
            [error shouldBeNil];
            resultsOfFetch = results;
        }];
        
        // Make a relationship
        // add some related objects
        NSManagedObject *interest1 = [NSEntityDescription insertNewObjectForEntityForName:@"Interest" inManagedObjectContext:moc];
        [interest1 setValue:[interest1 assignObjectId] forKey:[interest1 primaryKeyField]];
        [interest1 setValue:@"interest1" forKey:@"name"];
        
        NSManagedObject *interest2 = [NSEntityDescription insertNewObjectForEntityForName:@"Interest" inManagedObjectContext:moc];
        [interest2 setValue:[interest2 assignObjectId] forKey:[interest2 primaryKeyField]];
        [interest2 setValue:@"interest2" forKey:@"name"];
        
        // save them to the server
        [SMCoreDataIntegrationTestHelpers executeSynchronousSave:moc withBlock:^(NSError *error) {
            [error shouldBeNil];
        }];
        
        sleep(SLEEP_TIME);
        
        [[resultsOfFetch objectAtIndex:0] addInterests:[NSSet setWithObjects:interest1, interest2, nil]];
        
        [SMCoreDataIntegrationTestHelpers executeSynchronousSave:moc withBlock:^(NSError *error) {
            [error shouldBeNil];
        }];
        
        sleep(SLEEP_TIME);
        
        // Fetch all persons again to create nil references
        [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:moc withRequest:[SMCoreDataIntegrationTestHelpers makePersonFetchRequest:nil context:moc] andBlock:^(NSArray *results, NSError *error) {
            resultsOfFetch = results;
        }];
    });
    afterEach(^{
        // delete
        [cds setFetchPolicy:SMFetchPolicyNetworkOnly];
        [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:moc withRequest:[SMCoreDataIntegrationTestHelpers makeInterestFetchRequest:nil context:moc] andBlock:^(NSArray *results, NSError *error) {
            [results enumerateObjectsUsingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
                [moc deleteObject:obj];
            }];
        }];
        [SMCoreDataIntegrationTestHelpers executeSynchronousSave:moc withBlock:^(NSError *error) {
            [error shouldBeNil];
        }];
        [SMIntegrationTestHelpers destroyAllForFixturesNamed:fixturesToLoad];
        SM_CACHE_ENABLED = NO;
        
        sleep(SLEEP_TIME);
    });
    it(@"works", ^{
        
        [cds setFetchPolicy:SMFetchPolicyTryCacheElseNetwork];
        
        // No longer hold because of cache writing
        //[[cds should] receive:@selector(performQuery:options:successCallbackQueue:failureCallbackQueue:onSuccess:onFailure:) withCount:1];
        
        // Fetch that entity, should not throw an exception
        [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:moc withRequest:[SMCoreDataIntegrationTestHelpers makeInterestFetchRequest:nil context:moc] andBlock:^(NSArray *results, NSError *error) {
            [error shouldBeNil];
            [[theValue([results count]) should] equal:theValue(2)];
        }];
    });
});

describe(@"Testing cache using Entity with a GeoPoint attribute", ^{
    __block NSManagedObjectContext *moc = nil;
    __block NSManagedObject *geoObject = nil;
    __block NSManagedObject *geoObject2 = nil;
    __block SMClient *client = nil;
    __block SMCoreDataStore *cds = nil;
    __block NSDictionary *location = nil;
    beforeEach(^{
        SM_CACHE_ENABLED = YES;
        //SM_CORE_DATA_DEBUG = YES;
        client = [SMIntegrationTestHelpers defaultClient];
        [SMClient setDefaultClient: client];
        [SMCoreDataIntegrationTestHelpers removeSQLiteDatabaseAndMapsWithPublicKey:client.publicKey];
        NSBundle *classBundle = [NSBundle bundleForClass:[self class]];
        NSURL *modelURL = [classBundle URLForResource:@"SMCoreDataIntegrationTest" withExtension:@"momd"];
        NSManagedObjectModel *aModel = [[NSManagedObjectModel alloc] initWithContentsOfURL:modelURL];
        cds = [client coreDataStoreWithManagedObjectModel:aModel];
        [cds setFetchPolicy:SMFetchPolicyNetworkOnly];
        moc = [cds contextForCurrentThread];
        [[client.session.networkMonitor stubAndReturn:theValue(1)] currentNetworkStatus];
        
        geoObject = [NSEntityDescription insertNewObjectForEntityForName:@"Random" inManagedObjectContext:moc];
        
        NSNumber *lat = [NSNumber numberWithDouble:37.77215879638275];
        NSNumber *lon = [NSNumber numberWithDouble:-122.4064476357965];
        
        location = [NSDictionary dictionaryWithObjectsAndKeys:
                    lat
                    ,@"lat"
                    ,lon
                    ,@"lon", nil];
        
        NSData *data = [NSKeyedArchiver archivedDataWithRootObject:location];
        
        [geoObject setValue:data forKey:@"geopoint"];
        [geoObject setValue:@"StackMob" forKey:@"name"];
        [geoObject setValue:[geoObject assignObjectId] forKey:[geoObject primaryKeyField]];
        
        geoObject2 = [NSEntityDescription insertNewObjectForEntityForName:@"Random" inManagedObjectContext:moc];
        NSNumber *lat2 = [NSNumber numberWithDouble:42.280373];
        NSNumber *lon2 = [NSNumber numberWithDouble:-71.416669];
        
        NSDictionary *location2 = [NSDictionary dictionaryWithObjectsAndKeys:
                                   lat2
                                   ,@"lat"
                                   ,lon2
                                   ,@"lon", nil];
        
        NSData *data2 = [NSKeyedArchiver archivedDataWithRootObject:location2];
        [geoObject2 setValue:data2 forKey:@"geopoint"];
        [geoObject2 setValue:@"Framingahm" forKey:@"name"];
        [geoObject2 setValue:[geoObject2 assignObjectId] forKey:[geoObject2 primaryKeyField]];
        
        
        [SMCoreDataIntegrationTestHelpers executeSynchronousSave:moc withBlock:^(NSError *error) {
            if (error != nil) {
                DLog(@"Error userInfo is %@", [error userInfo]);
                [error shouldBeNil];
            }
        }];
        
        sleep(SLEEP_TIME);
    });
    afterEach(^{
        [cds setFetchPolicy:SMFetchPolicyNetworkOnly];
        [[client.session.networkMonitor stubAndReturn:theValue(1)] currentNetworkStatus];
        [moc deleteObject:geoObject];
        [moc deleteObject:geoObject2];
        [SMCoreDataIntegrationTestHelpers executeSynchronousSave:moc withBlock:^(NSError *error) {
            if (error != nil) {
                DLog(@"Error userInfo is %@", [error userInfo]);
                [error shouldBeNil];
            }
        }];
        SM_CACHE_ENABLED = NO;
    });
    it(@"Will prevent SMPredicate query on read cache", ^{
        
        NSEntityDescription *entity = [NSEntityDescription entityForName:@"Random" inManagedObjectContext:moc];
        NSFetchRequest *fetchRequest = [[NSFetchRequest alloc] init];
        [fetchRequest setEntity:entity];
        
        // Fisherman's Wharf
        CLLocationCoordinate2D coordinate;
        coordinate.latitude = 37.810317;
        coordinate.longitude = -122.418167;
        
        SMPredicate *predicate = [SMPredicate predicateWhere:@"geopoint" isWithin:3.5
                                                     milesOf:coordinate];
        [fetchRequest setPredicate:predicate];
        
        
        
        [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:moc withRequest:fetchRequest andBlock:^(NSArray *results, NSError *error) {
            if (error != nil) {
                [error shouldBeNil];
            }
            [[theValue([results count]) should] equal:theValue(1)];
            
            
            NSData *comparisonData = [[results objectAtIndex:0] valueForKey:@"geopoint"];
            NSDictionary *comparisonDictionary = [NSKeyedUnarchiver unarchiveObjectWithData:comparisonData];
            
            [[comparisonDictionary should] equal:location];
        }];
        
        [cds setFetchPolicy:SMFetchPolicyCacheOnly];
        
        [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:moc withRequest:fetchRequest andBlock:^(NSArray *results, NSError *error) {
            if (error != nil) {
                [error shouldBeNil];
            }            
            [[theValue([results count]) should] equal:theValue(0)];
        }];
    });
    it(@"Will successfully read with an NSPredicate on read cache", ^{
        
        NSEntityDescription *entity = [NSEntityDescription entityForName:@"Random" inManagedObjectContext:moc];
        NSFetchRequest *fetchRequest = [[NSFetchRequest alloc] init];
        [fetchRequest setEntity:entity];
        
        SMPredicate *predicate = (SMPredicate *)[NSPredicate predicateWithFormat:@"name == %@", @"StackMob"];
        [fetchRequest setPredicate:predicate];
        
        [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:moc withRequest:fetchRequest andBlock:^(NSArray *results, NSError *error) {
            if (error != nil) {
                [error shouldBeNil];
            }
            [[theValue([results count]) should] equal:theValue(1)];
            
            
            NSData *comparisonData = [[results objectAtIndex:0] valueForKey:@"geopoint"];
            NSDictionary *comparisonDictionary = [NSKeyedUnarchiver unarchiveObjectWithData:comparisonData];
            
            [[comparisonDictionary should] equal:location];
        }];
        
        [cds setFetchPolicy:SMFetchPolicyCacheOnly];
        
        [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:moc withRequest:fetchRequest andBlock:^(NSArray *results, NSError *error) {
            if (error != nil) {
                [error shouldBeNil];
            }
            [[theValue([results count]) should] equal:theValue(1)];
            
            
            NSData *comparisonData = [[results objectAtIndex:0] valueForKey:@"geopoint"];
            NSDictionary *comparisonDictionary = [NSKeyedUnarchiver unarchiveObjectWithData:comparisonData];
            
            [[comparisonDictionary should] equal:location];
        }];
        
    });
    it(@"Will prevent compound query with SMPredicate on read cache", ^{
        
        NSEntityDescription *entity = [NSEntityDescription entityForName:@"Random" inManagedObjectContext:moc];
        NSFetchRequest *fetchRequest = [[NSFetchRequest alloc] init];
        [fetchRequest setEntity:entity];
        
        // Fisherman's Wharf
        CLLocationCoordinate2D coordinate;
        coordinate.latitude = 37.810317;
        coordinate.longitude = -122.418167;
        
        SMPredicate *geoPredicate = [SMPredicate predicateWhere:@"geopoint" isWithin:1000
                                                        milesOf:coordinate];
        NSPredicate *predicate = [NSPredicate predicateWithFormat:@"name == %@", @"StackMob"];
        NSArray *predicates = [NSArray arrayWithObjects:geoPredicate, predicate, nil];
        
        NSPredicate *compoundPredicate =[NSCompoundPredicate andPredicateWithSubpredicates:predicates];
        [fetchRequest setPredicate:compoundPredicate];
        
        [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:moc withRequest:fetchRequest andBlock:^(NSArray *results, NSError *error) {
            if (error != nil) {
                [error shouldBeNil];
            }
            [[theValue([results count]) should] equal:theValue(1)];
            
            
            NSData *comparisonData = [[results objectAtIndex:0] valueForKey:@"geopoint"];
            NSDictionary *comparisonDictionary = [NSKeyedUnarchiver unarchiveObjectWithData:comparisonData];
            
            [[comparisonDictionary should] equal:location];
        }];
        
        [cds setFetchPolicy:SMFetchPolicyCacheOnly];
        
        [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:moc withRequest:fetchRequest andBlock:^(NSArray *results, NSError *error) {
            if (error != nil) {
                [error shouldBeNil];
            }            
            [[theValue([results count]) should] equal:theValue(0)];
        }];
    });
    
    it(@"Will prevent compound query with SMPredicate (embedded two levels deep) on read cache", ^{
        
        NSEntityDescription *entity = [NSEntityDescription entityForName:@"Random" inManagedObjectContext:moc];
        NSFetchRequest *fetchRequest = [[NSFetchRequest alloc] init];
        [fetchRequest setEntity:entity];
        
        // Fisherman's Wharf
        CLLocationCoordinate2D coordinate;
        coordinate.latitude = 37.810317;
        coordinate.longitude = -122.418167;
        
        SMPredicate *geoPredicate = [SMPredicate predicateWhere:@"geopoint" isWithin:1000
                                                        milesOf:coordinate];
        NSPredicate *predicate = [NSPredicate predicateWithFormat:@"name == %@", @"StackMob"];
        NSArray *predicates = [NSArray arrayWithObjects:geoPredicate, predicate, nil];
        
        NSPredicate *compoundPredicate =[NSCompoundPredicate andPredicateWithSubpredicates:predicates];
        
        NSPredicate *predicate2 = [NSPredicate predicateWithFormat:@"name == %@", @"StackMob"];
        NSArray *predicates2 = [NSArray arrayWithObjects:predicate2, compoundPredicate, nil];
        
        NSPredicate *compoundPredicate2 =[NSCompoundPredicate andPredicateWithSubpredicates:predicates2];
        [fetchRequest setPredicate:compoundPredicate2];
        
        [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:moc withRequest:fetchRequest andBlock:^(NSArray *results, NSError *error) {
            if (error != nil) {
                [error shouldBeNil];
            }
            [[theValue([results count]) should] equal:theValue(1)];
            
            
            NSData *comparisonData = [[results objectAtIndex:0] valueForKey:@"geopoint"];
            NSDictionary *comparisonDictionary = [NSKeyedUnarchiver unarchiveObjectWithData:comparisonData];
            
            [[comparisonDictionary should] equal:location];
        }];
        
        [cds setFetchPolicy:SMFetchPolicyCacheOnly];
        
        [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:moc withRequest:fetchRequest andBlock:^(NSArray *results, NSError *error) {
            if (error != nil) {
                [error shouldBeNil];
            }
            [[theValue([results count]) should] equal:theValue(0)];
        }];
    });
});

// MISC TESTS

/*
 describe(@"testing", ^{
 __block SMClient *client = nil;
 __block SMCoreDataStore *cds = nil;
 __block NSManagedObjectContext *moc = nil;
 __block NSArray *fixturesToLoad;
 __block NSDictionary *fixtures;
 beforeEach(^{
 SM_CACHE_ENABLED = YES;
 //SM_CORE_DATA_DEBUG = YES;
 
 //fixturesToLoad = [NSArray arrayWithObjects:@"person", nil];
 //fixtures = [SMIntegrationTestHelpers loadFixturesNamed:fixturesToLoad];
 client = [SMIntegrationTestHelpers defaultClient];
 [SMClient setDefaultClient:client];
 [SMCoreDataIntegrationTestHelpers removeSQLiteDatabaseAndMapsWithPublicKey:client.publicKey];
 NSBundle *classBundle = [NSBundle bundleForClass:[self class]];
 NSURL *modelURL = [classBundle URLForResource:@"SMCoreDataIntegrationTest" withExtension:@"momd"];
 NSManagedObjectModel *aModel = [[NSManagedObjectModel alloc] initWithContentsOfURL:modelURL];
 cds = [client coreDataStoreWithManagedObjectModel:aModel];
 moc = [cds contextForCurrentThread];
 
 // create an object
 NSDictionary *dictionary = [NSDictionary dictionaryWithObjectsAndKeys:@"1234", @"todo_id", @"new todo", @"title", nil];
 syncWithSemaphore(^(dispatch_semaphore_t semaphore) {
 [[client dataStore] createObject:dictionary inSchema:@"todo" onSuccess:^(NSDictionary *object, NSString *schema) {
 NSLog(@"successful create");
 syncReturn(semaphore);
 } onFailure:^(NSError *error, NSDictionary *object, NSString *schema) {
 NSLog(@"failure creating: %@", error);
 syncReturn(semaphore);
 }];
 });
 
 });
 afterEach(^{
 [cds setFetchPolicy:SMFetchPolicyNetworkOnly];
 syncWithSemaphore(^(dispatch_semaphore_t semaphore) {
 [[client dataStore] deleteObjectId:@"1234" inSchema:@"todo" onSuccess:^(NSString *objectId, NSString *schema) {
 NSLog(@"successful delete");
 syncReturn(semaphore);
 } onFailure:^(NSError *error, NSString *objectId, NSString *schema) {
 NSLog(@"error deleting: %@", error);
 syncReturn(semaphore);
 }];
 });
 //[SMIntegrationTestHelpers destroyAllForFixturesNamed:fixturesToLoad];
 SM_CACHE_ENABLED = NO;
 });
 it(@"tests", ^{
 NSPredicate *pred = [NSPredicate predicateWithFormat:@"todoId == '1234'"];
 NSFetchRequest *fetch = [[NSFetchRequest alloc] initWithEntityName:@"Todo"];
 [fetch setPredicate:pred];
 NSError *error = nil;
 NSArray *results = [moc executeFetchRequestAndWait:fetch error:&error];
 NSLog(@"results are %@", results);
 [error shouldBeNil];
 
 error = nil;
 results = [moc executeFetchRequestAndWait:fetch error:&error];
 NSLog(@"results are %@", results);
 [error shouldBeNil];
 });
 });
 */

/*
 describe(@"testing2", ^{
 __block SMClient *client = nil;
 __block SMCoreDataStore *cds = nil;
 __block NSManagedObjectContext *moc = nil;
 beforeEach(^{
 SM_CACHE_ENABLED = YES;
 SM_CORE_DATA_DEBUG = YES;
 
 client = [SMIntegrationTestHelpers defaultClient];
 [SMClient setDefaultClient:client];
 [SMCoreDataIntegrationTestHelpers removeSQLiteDatabaseAndMapsWithPublicKey:client.publicKey];
 NSBundle *classBundle = [NSBundle bundleForClass:[self class]];
 NSURL *modelURL = [classBundle URLForResource:@"SMCoreDataIntegrationTest" withExtension:@"momd"];
 NSManagedObjectModel *aModel = [[NSManagedObjectModel alloc] initWithContentsOfURL:modelURL];
 cds = [client coreDataStoreWithManagedObjectModel:aModel];
 moc = [cds contextForCurrentThread];
 
 NSManagedObject *todo = [NSEntityDescription insertNewObjectForEntityForName:@"Todo" inManagedObjectContext:moc];
 [todo setValue:@"bob" forKey:@"title"];
 [todo setValue:@"1234" forKey:[todo primaryKeyField]];
 
 NSManagedObject *category = [NSEntityDescription insertNewObjectForEntityForName:@"Category" inManagedObjectContext:moc];
 [category setValue:@"new" forKey:@"name"];
 [category setValue:@"primarykey" forKey:[category primaryKeyField]];
 
 [todo setValue:category forKey:@"category"];
 
 NSError *error = nil;
 BOOL success = [moc saveAndWait:&error];
 [[theValue(success) should] beYes];
 
 });
 afterEach(^{
 [cds setFetchPolicy:SMFetchPolicyNetworkOnly];
 NSFetchRequest *todoFetch = [[NSFetchRequest alloc] initWithEntityName:@"Todo"];
 [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:moc withRequest:todoFetch andBlock:^(NSArray *results, NSError *error) {
 [error shouldBeNil];
 [moc deleteObject:[results objectAtIndex:0]];
 }];
 NSFetchRequest *categoryFetch = [[NSFetchRequest alloc] initWithEntityName:@"Category"];
 [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:moc withRequest:categoryFetch andBlock:^(NSArray *results, NSError *error) {
 [error shouldBeNil];
 [moc deleteObject:[results objectAtIndex:0]];
 }];
 
 NSError *error = nil;
 BOOL success = [moc saveAndWait:&error];
 [[theValue(success) should] beYes];
 
 SM_CACHE_ENABLED = NO;
 });
 it(@"tests", ^{
 [moc reset];
 
 NSPredicate *pred = [NSPredicate predicateWithFormat:@"todoId == '1234'"];
 NSFetchRequest *fetch = [[NSFetchRequest alloc] initWithEntityName:@"Todo"];
 [fetch setPredicate:pred];
 NSError *error = nil;
 NSArray *results = [moc executeFetchRequestAndWait:fetch error:&error];
 NSLog(@"results are %@", results);
 [error shouldBeNil];
 
 NSManagedObject *todo = [results objectAtIndex:0];
 
 NSManagedObject *category = [todo valueForKey:@"category"];
 NSLog(@"category is %@", category);
 NSString *categoryTitle = [category valueForKey:@"name"];
 NSLog(@"title is %@", categoryTitle);
 });
 });
 */


/*
describe(@"calls to save when not online", ^{
    __block SMClient *client = nil;
    __block SMCoreDataStore *cds = nil;
    __block NSManagedObjectContext *moc = nil;
    beforeEach(^{
        SM_CORE_DATA_DEBUG = YES;
        client = [SMIntegrationTestHelpers defaultClient];
        [SMClient setDefaultClient:client];
        NSBundle *classBundle = [NSBundle bundleForClass:[self class]];
        NSURL *modelURL = [classBundle URLForResource:@"SMCoreDataIntegrationTest" withExtension:@"momd"];
        NSManagedObjectModel *aModel = [[NSManagedObjectModel alloc] initWithContentsOfURL:modelURL];
        cds = [client coreDataStoreWithManagedObjectModel:aModel];
        moc = [cds contextForCurrentThread];
    });
    it(@"should fail with appropriate error during insert", ^{
        [[client.session.networkMonitor stubAndReturn:theValue(0)] currentNetworkStatus];
        
        NSManagedObject *superpower = [NSEntityDescription insertNewObjectForEntityForName:@"Superpower" inManagedObjectContext:moc];
        [superpower setValue:[superpower assignObjectId] forKey:[superpower primaryKeyField]];
        [superpower setValue:@"superpower" forKey:@"name"];
        
        // save them to the server
        [SMCoreDataIntegrationTestHelpers executeSynchronousSave:moc withBlock:^(NSError *error) {
            [error shouldNotBeNil];
            [[theValue([error code]) should] equal:theValue(SMErrorNetworkNotReachable)];
        }];
        
        
    });
    it(@"should fail with appropriate error during update", ^{
        [[client.session.networkMonitor stubAndReturn:theValue(1)] currentNetworkStatus];
        
        NSManagedObject *superpower = [NSEntityDescription insertNewObjectForEntityForName:@"Superpower" inManagedObjectContext:moc];
        [superpower setValue:[superpower assignObjectId] forKey:[superpower primaryKeyField]];
        [superpower setValue:@"superpower" forKey:@"name"];
        
        [SMCoreDataIntegrationTestHelpers executeSynchronousSave:moc withBlock:^(NSError *error) {
            [error shouldBeNil];
        }];
        
        [[client.session.networkMonitor stubAndReturn:theValue(0)] currentNetworkStatus];
        
        [superpower setValue:@"new name" forKey:@"name"];
        
        [SMCoreDataIntegrationTestHelpers executeSynchronousSave:moc withBlock:^(NSError *error) {
            [error shouldNotBeNil];
            [[theValue([error code]) should] equal:theValue(SMErrorNetworkNotReachable)];
        }];
        
        [[client.session.networkMonitor stubAndReturn:theValue(1)] currentNetworkStatus];
        
        [moc deleteObject:superpower];
        
        [SMCoreDataIntegrationTestHelpers executeSynchronousSave:moc withBlock:^(NSError *error) {
            [error shouldBeNil];
        }];
    });
    
    it(@"should fail with appropriate error during update with to-one", ^{
        [[client.session.networkMonitor stubAndReturn:theValue(1)] currentNetworkStatus];
        
        NSManagedObject *person = [NSEntityDescription insertNewObjectForEntityForName:@"Person" inManagedObjectContext:moc];
        [person setValue:[person assignObjectId] forKey:[person primaryKeyField]];
        [person setValue:@"bob" forKey:@"first_name"];
        
        NSManagedObject *superpower = [NSEntityDescription insertNewObjectForEntityForName:@"Superpower" inManagedObjectContext:moc];
        [superpower setValue:[superpower assignObjectId] forKey:[superpower primaryKeyField]];
        [superpower setValue:@"superpower" forKey:@"name"];
        
        [SMCoreDataIntegrationTestHelpers executeSynchronousSave:moc withBlock:^(NSError *error) {
            [error shouldBeNil];
        }];
        
        [[client.session.networkMonitor stubAndReturn:theValue(0)] currentNetworkStatus];
        
        [person setValue:superpower forKey:@"superpower"];
        
        [SMCoreDataIntegrationTestHelpers executeSynchronousSave:moc withBlock:^(NSError *error) {
            [error shouldNotBeNil];
            [[theValue([error code]) should] equal:theValue(SMErrorNetworkNotReachable)];
        }];
        
        [[client.session.networkMonitor stubAndReturn:theValue(1)] currentNetworkStatus];
        
        [moc deleteObject:person];
        [moc deleteObject:superpower];
        
        [SMCoreDataIntegrationTestHelpers executeSynchronousSave:moc withBlock:^(NSError *error) {
            [error shouldBeNil];
        }];
    });
    
    it(@"should fail with appropriate error during update with to-many", ^{
        [[client.session.networkMonitor stubAndReturn:theValue(1)] currentNetworkStatus];
        
        NSManagedObject *person = [NSEntityDescription insertNewObjectForEntityForName:@"Person" inManagedObjectContext:moc];
        [person setValue:[person assignObjectId] forKey:[person primaryKeyField]];
        [person setValue:@"bob" forKey:@"first_name"];
        
        NSManagedObject *interest1 = [NSEntityDescription insertNewObjectForEntityForName:@"Interest" inManagedObjectContext:moc];
        [interest1 setValue:[interest1 assignObjectId] forKey:[interest1 primaryKeyField]];
        [interest1 setValue:@"interest1" forKey:@"name"];
        
        NSManagedObject *interest2 = [NSEntityDescription insertNewObjectForEntityForName:@"Interest" inManagedObjectContext:moc];
        [interest2 setValue:[interest2 assignObjectId] forKey:[interest2 primaryKeyField]];
        [interest2 setValue:@"interest2" forKey:@"name"];
        
        [SMCoreDataIntegrationTestHelpers executeSynchronousSave:moc withBlock:^(NSError *error) {
            [error shouldBeNil];
        }];
        
        [[client.session.networkMonitor stubAndReturn:theValue(0)] currentNetworkStatus];
        
        [person setValue:[NSSet setWithObjects:interest1, interest2, nil] forKey:@"interests"];
        
        [SMCoreDataIntegrationTestHelpers executeSynchronousSave:moc withBlock:^(NSError *error) {
            [error shouldNotBeNil];
            [[theValue([error code]) should] equal:theValue(SMErrorNetworkNotReachable)];
        }];
        
        [[client.session.networkMonitor stubAndReturn:theValue(1)] currentNetworkStatus];
        
        [moc deleteObject:person];
        [moc deleteObject:interest1];
        [moc deleteObject:interest2];
        
        [SMCoreDataIntegrationTestHelpers executeSynchronousSave:moc withBlock:^(NSError *error) {
            [error shouldBeNil];
        }];
    });
    it(@"should fail with appropriate error during delete", ^{
        [[client.session.networkMonitor stubAndReturn:theValue(1)] currentNetworkStatus];
        
        NSManagedObject *superpower = [NSEntityDescription insertNewObjectForEntityForName:@"Superpower" inManagedObjectContext:moc];
        [superpower setValue:[superpower assignObjectId] forKey:[superpower primaryKeyField]];
        [superpower setValue:@"superpower" forKey:@"name"];
        
        [SMCoreDataIntegrationTestHelpers executeSynchronousSave:moc withBlock:^(NSError *error) {
            [error shouldBeNil];
        }];
        
        [[client.session.networkMonitor stubAndReturn:theValue(0)] currentNetworkStatus];
        
        [moc deleteObject:superpower];
        
        [SMCoreDataIntegrationTestHelpers executeSynchronousSave:moc withBlock:^(NSError *error) {
            [error shouldNotBeNil];
            [[theValue([error code]) should] equal:theValue(SMErrorNetworkNotReachable)];
        }];
        
        [[client.session.networkMonitor stubAndReturn:theValue(1)] currentNetworkStatus];
        
        [SMCoreDataIntegrationTestHelpers executeSynchronousSave:moc withBlock:^(NSError *error) {
            [error shouldBeNil];
        }];
    });
     
});


describe(@"returning proper errors from reads", ^{
    __block SMClient *client = nil;
    __block SMCoreDataStore *cds = nil;
    __block NSManagedObjectContext *moc = nil;
    beforeEach(^{
        SM_CORE_DATA_DEBUG = YES;
        client = [SMIntegrationTestHelpers defaultClient];
        NSBundle *classBundle = [NSBundle bundleForClass:[self class]];
        NSURL *modelURL = [classBundle URLForResource:@"SMCoreDataIntegrationTest" withExtension:@"momd"];
        NSManagedObjectModel *aModel = [[NSManagedObjectModel alloc] initWithContentsOfURL:modelURL];
        cds = [client coreDataStoreWithManagedObjectModel:aModel];
        moc = [cds contextForCurrentThread];
    });
    it(@"new values for object on save with a 401", ^{
        [[client.session.networkMonitor stubAndReturn:theValue(1)] currentNetworkStatus];
        
        NSManagedObject *newMO = [NSEntityDescription insertNewObjectForEntityForName:@"Getpermission" inManagedObjectContext:moc];
        [newMO setValue:[newMO assignObjectId] forKey:[newMO primaryKeyField]];
        [newMO setValue:@"bob" forKey:@"name"];
        
        [SMCoreDataIntegrationTestHelpers executeSynchronousSave:moc withBlock:^(NSError *error) {
            [error shouldBeNil];
        }];
        
        [newMO setValue:@"jack" forKey:@"name"];
        
        __block NSError *anError = nil;
        [moc performBlockAndWait:^{
            BOOL savesuccess = [moc save:&anError];
            if (!savesuccess) {
                NSLog(@"error is %@", anError);
            }
        }];
        
        [moc deleteObject:newMO];
        [SMCoreDataIntegrationTestHelpers executeSynchronousSave:moc withBlock:^(NSError *error) {
            [error shouldBeNil];
        }];
        
    });
});
 */
/*
describe(@"cache enabling and disabling", ^{
    __block SMClient *client = nil;
    __block SMCoreDataStore *cds = nil;
    __block NSManagedObjectContext *moc = nil;
    beforeEach(^{
        SM_CORE_DATA_DEBUG = YES;
        [SMCoreDataIntegrationTestHelpers removeSQLiteDatabaseAndMaps];
        client = [SMIntegrationTestHelpers defaultClient];
        [SMClient setDefaultClient:client];
        NSBundle *classBundle = [NSBundle bundleForClass:[self class]];
        NSURL *modelURL = [classBundle URLForResource:@"SMCoreDataIntegrationTest" withExtension:@"momd"];
        NSManagedObjectModel *aModel = [[NSManagedObjectModel alloc] initWithContentsOfURL:modelURL];
        cds = [client coreDataStoreWithManagedObjectModel:aModel];
        moc = [cds contextForCurrentThread];
    });
    afterEach(^{
        [cds enableCache];
        [[client.session.networkMonitor stubAndReturn:theValue(1)] currentNetworkStatus];
        __block NSManagedObject *fetchedPerson = nil;
        // Fetch person
        [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:moc withRequest:[SMCoreDataIntegrationTestHelpers makePersonFetchRequest:[NSPredicate predicateWithFormat:@"first_name == 'bob'"]] andBlock:^(NSArray *results, NSError *error) {
            [[theValue([results count]) should] equal:theValue(1)];
            
            fetchedPerson = [results objectAtIndex:0];
            
        }];
        
        [moc deleteObject:fetchedPerson];
        
        [SMCoreDataIntegrationTestHelpers executeSynchronousSave:moc withBlock:^(NSError *error) {
            [error shouldBeNil];
        }];
        
    });
    it(@"cache should not be on by default", ^{
        
        [[theValue([cds cacheIsEnabled]) should] beNo];
        
        [[client.session.networkMonitor stubAndReturn:theValue(1)] currentNetworkStatus];
        // Create person
        NSManagedObject *person = [NSEntityDescription insertNewObjectForEntityForName:@"Person" inManagedObjectContext:moc];
        [person setValue:[person assignObjectId] forKey:[person primaryKeyField]];
        [person setValue:@"bob" forKey:@"first_name"];
        
        [SMCoreDataIntegrationTestHelpers executeSynchronousSave:moc withBlock:^(NSError *error) {
            [error shouldBeNil];
        }];
        __block NSManagedObject *fetchedPerson = nil;
        // Fetch person
        [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:moc withRequest:[SMCoreDataIntegrationTestHelpers makePersonFetchRequest:[NSPredicate predicateWithFormat:@"first_name == 'bob'"]] andBlock:^(NSArray *results, NSError *error) {
            [[theValue([results count]) should] equal:theValue(1)];
            
            fetchedPerson = [results objectAtIndex:0];
            
        }];
        
        // fetchedPerson should be a fault
        [[theValue([fetchedPerson isFault]) should] beYes];
        
        // being online, fault fill should cause a network call
        [[cds should] receive:@selector(readObjectWithId:inSchema:options:successCallbackQueue:failureCallbackQueue:onSuccess:onFailure:) withCount:1];
        NSString *name = [fetchedPerson valueForKey:@"first_name"];
        [[name should] equal:@"bob"];
        
        
    });
    it(@"can turn the cache on", ^{
        
        [[theValue([cds cacheIsEnabled]) should] beNo];
        
        [cds enableCache];
        
        [[theValue([cds cacheIsEnabled])  should] beYes];
        
        [[client.session.networkMonitor stubAndReturn:theValue(1)] currentNetworkStatus];
        // Create person
        NSManagedObject *person = [NSEntityDescription insertNewObjectForEntityForName:@"Person" inManagedObjectContext:moc];
        [person setValue:[person assignObjectId] forKey:[person primaryKeyField]];
        [person setValue:@"bob" forKey:@"first_name"];
        
        [SMCoreDataIntegrationTestHelpers executeSynchronousSave:moc withBlock:^(NSError *error) {
            [error shouldBeNil];
        }];
        __block NSManagedObject *fetchedPerson = nil;
        // Fetch person
        [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:moc withRequest:[SMCoreDataIntegrationTestHelpers makePersonFetchRequest:[NSPredicate predicateWithFormat:@"first_name == 'bob'"]] andBlock:^(NSArray *results, NSError *error) {
            [[theValue([results count]) should] equal:theValue(1)];
            
            fetchedPerson = [results objectAtIndex:0];
            
        }];
        
        // fetchedPerson should be a fault
        [[theValue([fetchedPerson isFault]) should] beYes];
        
        // being online, fault fill should cause a network call
        [[cds should] receive:@selector(readObjectWithId:inSchema:options:successCallbackQueue:failureCallbackQueue:onSuccess:onFailure:) withCount:0];
        NSString *name = [fetchedPerson valueForKey:@"first_name"];
        [[name should] equal:@"bob"];
        
        
    });

    it(@"can turn the cache on and off", ^{
        
        [[theValue([cds cacheIsEnabled]) should] beNo];
        
        [cds enableCache];
        
        [[theValue([cds cacheIsEnabled])  should] beYes];
        
        [[client.session.networkMonitor stubAndReturn:theValue(1)] currentNetworkStatus];
        // Create person
        NSManagedObject *person = [NSEntityDescription insertNewObjectForEntityForName:@"Person" inManagedObjectContext:moc];
        [person setValue:[person assignObjectId] forKey:[person primaryKeyField]];
        [person setValue:@"bob" forKey:@"first_name"];
        
        [SMCoreDataIntegrationTestHelpers executeSynchronousSave:moc withBlock:^(NSError *error) {
            [error shouldBeNil];
        }];
        __block NSManagedObject *fetchedPerson = nil;
        // Fetch person
        [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:moc withRequest:[SMCoreDataIntegrationTestHelpers makePersonFetchRequest:[NSPredicate predicateWithFormat:@"first_name == 'bob'"]] andBlock:^(NSArray *results, NSError *error) {
            [[theValue([results count]) should] equal:theValue(1)];
            
            fetchedPerson = [results objectAtIndex:0];
            
        }];
        
        // fetchedPerson should be a fault
        [[theValue([fetchedPerson isFault]) should] beYes];
        
        // being online, fault fill should cause a network call
        NSString *name = [fetchedPerson valueForKey:@"first_name"];
        [[name should] equal:@"bob"];
        
        // DISABLE CACHE
        [moc reset];
        [moc.parentContext reset];
        [cds disableCache];
        
        [[theValue([cds cacheIsEnabled])  should] beNo];
        
        [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:moc withRequest:[SMCoreDataIntegrationTestHelpers makePersonFetchRequest:[NSPredicate predicateWithFormat:@"first_name == 'bob'"]] andBlock:^(NSArray *results, NSError *error) {
            [[theValue([results count]) should] equal:theValue(1)];
            
            fetchedPerson = [results objectAtIndex:0];
            
        }];
        
        // fetchedPerson should be a fault
        [[theValue([fetchedPerson isFault]) should] beYes];
        
        // being online, fault fill should cause a network call
        [[cds should] receive:@selector(readObjectWithId:inSchema:options:successCallbackQueue:failureCallbackQueue:onSuccess:onFailure:) withCount:1];
        name = [fetchedPerson valueForKey:@"first_name"];
        
        [[name should] equal:@"bob"];
        
    });
    
});
*/

// extra

/*
 it(@"To-Many relationship fault fill without internet when related object has been previously fetched returns properly", ^{
 __block NSManagedObject *jonObject = nil;
 
 [cds setFetchPolicy:SMFetchPolicyNetworkOnly];
 
 // fetch new object, which will fault
 [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:moc withRequest:[SMCoreDataIntegrationTestHelpers makePersonFetchRequest:[NSPredicate predicateWithFormat:@"first_name == 'Jon'"]] andBlock:^(NSArray *results, NSError *error) {
 [[theValue([results count]) should] equal:theValue(1)];
 jonObject = [results objectAtIndex:0];
 NSManagedObject *nullSuperpower = [jonObject valueForKey:@"superpower"];
 [nullSuperpower shouldBeNil];
 }];
 
 // add some related objects
 NSManagedObject *interest1 = [NSEntityDescription insertNewObjectForEntityForName:@"Interest" inManagedObjectContext:moc];
 [interest1 setValue:[interest1 assignObjectId] forKey:[interest1 primaryKeyField]];
 [interest1 setValue:@"interest1" forKey:@"name"];
 
 NSManagedObject *interest2 = [NSEntityDescription insertNewObjectForEntityForName:@"Interest" inManagedObjectContext:moc];
 [interest2 setValue:[interest2 assignObjectId] forKey:[interest2 primaryKeyField]];
 [interest2 setValue:@"interest2" forKey:@"name"];
 
 // save them to the server
 [SMCoreDataIntegrationTestHelpers executeSynchronousSave:moc withBlock:^(NSError *error) {
 [error shouldBeNil];
 }];
 
 // relate and save
 [jonObject setValue:[NSSet setWithObjects:interest1, interest2, nil] forKey:@"interests"];
 
 [SMCoreDataIntegrationTestHelpers executeSynchronousSave:moc withBlock:^(NSError *error) {
 [error shouldBeNil];
 }];
 
 [moc reset];
 
 [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:moc withRequest:[SMCoreDataIntegrationTestHelpers makePersonFetchRequest:[NSPredicate predicateWithFormat:@"first_name == 'Jon'"]] andBlock:^(NSArray *results, NSError *error) {
 [[theValue([results count]) should] equal:theValue(1)];
 jonObject = [results objectAtIndex:0];
 }];
 
 [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:moc withRequest:[SMCoreDataIntegrationTestHelpers makeInterestFetchRequest:nil] andBlock:^(NSArray *results, NSError *error) {
 [[theValue([results count]) should] equal:theValue(2)];
 }];
 
 [cds setFetchPolicy:SMFetchPolicyCacheOnly];
 
 NSArray *jonInterests = nil;
 @try {
 jonInterests = [[jonObject valueForKey:@"interests"] allObjects];
 }
 @catch (NSException *exception) {
 [exception shouldBeNil];
 }
 
 [jonInterests shouldNotBeNil];
 [[theValue([jonInterests count]) should] equal:theValue(2)];
 NSString *interestName = [[jonInterests objectAtIndex:0] valueForKey:@"name"];
 NSArray *interestsArray = [NSArray arrayWithObjects:@"interest1", @"interest2", nil];
 [[interestsArray should] contain:interestName];
 
 
 [cds setFetchPolicy:SMFetchPolicyNetworkOnly];
 [SMCoreDataIntegrationTestHelpers executeSynchronousFetch:moc withRequest:[SMCoreDataIntegrationTestHelpers makeInterestFetchRequest:nil] andBlock:^(NSArray *results, NSError *error) {
 [results enumerateObjectsUsingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
 [moc deleteObject:obj];
 }];
 }];
 [SMCoreDataIntegrationTestHelpers executeSynchronousSave:moc withBlock:^(NSError *error) {
 [error shouldBeNil];
 }];
 });
 */

SPEC_END